/*
 * Copyright (C) 2019-2023 Roger Clark, VK3KYY / G4KYF
 *                         Daniel Caujolle-Bert, F1RMB
 *
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer
 *    in the documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Use of this source code or binary releases for commercial purposes is strictly forbidden. This includes, without limitation,
 *    incorporation in a commercial product or incorporation into a product or project which allows commercial use.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _OPENGD77_UC1701_CHARSET_H_
#define _OPENGD77_UC1701_CHARSET_H_

#if defined(BUILD_FONT_TOOL) || defined(CHAR_CONSTANTS_ONLY)

#define CHARS_PER_FONT 224

/*
 *************************************
   REF: https://www.ascii-code.com/
 *************************************
 */
//                                              DEC         OCT
//                                            +----+-----+------+
//                                            |    |     |      |
#define FONT_CHAR_CAPITAL_S_WITH_CARON 		138 // Š   \212
#define FONT_CHAR_CAPITAL_Z_WITH_CARON 		142 // Ž   \232
#define FONT_CHAR_SMALL_S_WITH_CARON		154 // š   \216
#define FONT_CHAR_SMALL_Z_WITH_CARON 		158 // ž   \236
#define FONT_CHAR_CAPITAL_Y_WITH_DIAERESIS	159 // Ÿ   \237
							       
#define FONT_CHAR_CAPITAL_C_WITH_CARON 		127 // Č   \177
#define FONT_CHAR_SMALL_C_WITH_CARON 		128 // č   \200
#define FONT_CHAR_CAPITAL_C_WITH_ACUTE  	129 // Ć   \201
#define FONT_CHAR_SMALL_C_WITH_ACUTE		130 // ć   \202
#define FONT_CHAR_CAPITAL_A_WITH_OGONEK  	131 // Ą   \203
#define FONT_CHAR_SMALL_A_WITH_OGONEK 		132 // ą   \204
#define FONT_CHAR_CAPITAL_E_WITH_OGONEK 	133 // Ę   \205
#define FONT_CHAR_SMALL_E_WITH_OGONEK 		134 // ę   \206
#define FONT_CHAR_CAPITAL_L_WITH_STROKE 	135 // Ł   \207
#define FONT_CHAR_SMALL_L_WITH_STROKE 		136 // ł   \210
#define FONT_CHAR_CAPITAL_N_WITH_ACUTE 		137 // Ń   \211
#define FONT_CHAR_SMALL_N_WITH_ACUTE  		139 // ń   \213
#define FONT_CHAR_CAPITAL_S_WITH_ACUTE 		140 // Ś   \214
#define FONT_CHAR_SMALL_S_WITH_ACUTE 		141 // ś   \215
#define FONT_CHAR_CAPITAL_Z_WITH_ACUTE 		143 // Ź   \217
#define FONT_CHAR_SMALL_Z_WITH_ACUTE		144 // ź   \220
#define FONT_CHAR_CAPITAL_Z_WITH_DOT_ABOVE 	145 // Ż   \221
#define FONT_CHAR_SMALL_Z_WITH_DOT_ABOVE  	146 // ż   \222
#define FONT_CHAR_SMALL_I_DOTLESS  		147 // ı   \223
#define FONT_CHAR_CAPITAL_I_WITH_DOT  		148 // İ   \224
#define FONT_CHAR_S_CAPITAL_WITH_CEDILLA	149 // Ş   \225
#define FONT_CHAR_S_SMALL_WITH_CEDILLA  	150 // ş   \226
#define FONT_CHAR_G_CAPITAL_WITH_BREVE		151 // Ğ   \227
#define FONT_CHAR_G_SMALL_WITH_BREVE		152 // ğ   \230
#define FONT_CHAR_CAPITAL_R_WITH_CARON 		153 // Ř   \231
#define FONT_CHAR_SMALL_R_WITH_CARON 		155 // ř   \233
#define FONT_CHAR_CAPITAL_E_WITH_CARON 		156 // Ě   \234
#define FONT_CHAR_SMALL_E_WITH_CARON 		157 // ě   \235
#define FONT_CHAR_CAPITAL_U_WITH_RING_ABOVE     160 // Ů   \240
#define FONT_CHAR_SMALL_U_WITH_RING_ABOVE       161 // ů   \241

#define FONT_CHAR_CAPITAL_O_WITH_DOUBLE_ACUTE   162 // Ő   \242
#define FONT_CHAR_SMALL_O_WITH_DOUBLE_ACUTE     163 // ő   \243
#define FONT_CHAR_CAPITAL_U_WITH_DOUBLE_ACUTE   164 // Ű   \244
#define FONT_CHAR_SMALL_U_WITH_DOUBLE_ACUTE     165 // ű   \245

// This is the list of excluded characters offsets
uint8_t freePositions[] = {
     166, 167, 168,
     169, 170, 171, 172, 173, 174, 175, 177, 178, 179, 180, 181, 182, 183, 184,
     185, 186, 187, 188, 189, 190, 191, 198, 215, 222, 230, 247, 254
};

struct
{
     uint32_t   offset;
     uint8_t    offsetName[48];
     uint8_t    character[5];
     uint8_t    octal;
} char_hand_assigned[] = {
     // The 5 following chars exists in windows-1252 charmap, but we need this for
     // font array creation (the character in comments)
     {  FONT_CHAR_CAPITAL_S_WITH_CARON       , "FONT_CHAR_CAPITAL_S_WITH_CARON"       , { "Š" } , '\212' }, // Š
     {  FONT_CHAR_SMALL_S_WITH_CARON         , "FONT_CHAR_SMALL_S_WITH_CARON"         , { "š" } , '\232' }, // š
     {  FONT_CHAR_CAPITAL_Z_WITH_CARON       , "FONT_CHAR_CAPITAL_Z_WITH_CARON"       , { "Ž" } , '\216' }, // Ž
     {  FONT_CHAR_SMALL_Z_WITH_CARON         , "FONT_CHAR_SMALL_Z_WITH_CARON"         , { "ž" } , '\236' }, // ž
     {  FONT_CHAR_CAPITAL_Y_WITH_DIAERESIS   , "FONT_CHAR_CAPITAL_Y_WITH_DIAERESIS"   , { "Ÿ" } , '\237' }, // Ÿ
     // ---											    
     {  FONT_CHAR_CAPITAL_C_WITH_CARON       , "FONT_CHAR_CAPITAL_C_WITH_CARON"       , { "Č" } , '\177' }, // Č
     {  FONT_CHAR_SMALL_C_WITH_CARON         , "FONT_CHAR_SMALL_C_WITH_CARON"         , { "č" } , '\200' }, // č
     {  FONT_CHAR_CAPITAL_C_WITH_ACUTE       , "FONT_CHAR_CAPITAL_C_WITH_ACUTE"       , { "Ć" } , '\201' }, // Ć
     {  FONT_CHAR_SMALL_C_WITH_ACUTE         , "FONT_CHAR_SMALL_C_WITH_ACUTE"         , { "ć" } , '\202' }, // ć
     {  FONT_CHAR_CAPITAL_A_WITH_OGONEK      , "FONT_CHAR_CAPITAL_A_WITH_OGONEK"      , { "Ą" } , '\203' }, // Ą
     {  FONT_CHAR_SMALL_A_WITH_OGONEK        , "FONT_CHAR_SMALL_A_WITH_OGONEK"        , { "ą" } , '\204' }, // ą
     {  FONT_CHAR_CAPITAL_E_WITH_OGONEK      , "FONT_CHAR_CAPITAL_E_WITH_OGONEK"      , { "Ę" } , '\205' }, // Ę
     {  FONT_CHAR_SMALL_E_WITH_OGONEK        , "FONT_CHAR_SMALL_E_WITH_OGONEK"        , { "ę" } , '\206' }, // ę
     {  FONT_CHAR_CAPITAL_L_WITH_STROKE      , "FONT_CHAR_CAPITAL_L_WITH_STROKE"      , { "Ł" } , '\207' }, // Ł
     {  FONT_CHAR_SMALL_L_WITH_STROKE        , "FONT_CHAR_SMALL_L_WITH_STROKE"        , { "ł" } , '\210' }, // ł
     {  FONT_CHAR_CAPITAL_N_WITH_ACUTE       , "FONT_CHAR_CAPITAL_N_WITH_ACUTE"       , { "Ń" } , '\211' }, // Ń
     {  FONT_CHAR_SMALL_N_WITH_ACUTE         , "FONT_CHAR_SMALL_N_WITH_ACUTE"         , { "ń" } , '\213' }, // ń
     {  FONT_CHAR_CAPITAL_S_WITH_ACUTE       , "FONT_CHAR_CAPITAL_S_WITH_ACUTE"       , { "Ś" } , '\214' }, // Ś
     {  FONT_CHAR_SMALL_S_WITH_ACUTE         , "FONT_CHAR_SMALL_S_WITH_ACUTE"         , { "ś" } , '\215' }, // ś
     {  FONT_CHAR_CAPITAL_Z_WITH_ACUTE       , "FONT_CHAR_CAPITAL_Z_WITH_ACUTE"       , { "Ź" } , '\217' }, // Ź
     {  FONT_CHAR_SMALL_Z_WITH_ACUTE         , "FONT_CHAR_SMALL_Z_WITH_ACUTE"         , { "ź" } , '\220' }, // ź
     {  FONT_CHAR_CAPITAL_Z_WITH_DOT_ABOVE   , "FONT_CHAR_CAPITAL_Z_WITH_DOT_ABOVE"   , { "Ż" } , '\221' }, // Ż
     {  FONT_CHAR_SMALL_Z_WITH_DOT_ABOVE     , "FONT_CHAR_SMALL_Z_WITH_DOT_ABOVE"     , { "ż" } , '\222' }, // ż
     {  FONT_CHAR_SMALL_I_DOTLESS            , "FONT_CHAR_SMALL_I_DOTLESS"            , { "ı" } , '\223' }, // ı
     {  FONT_CHAR_CAPITAL_I_WITH_DOT         , "FONT_CHAR_CAPITAL_I_WITH_DOT"         , { "İ" } , '\224' }, // İ
     {  FONT_CHAR_S_CAPITAL_WITH_CEDILLA     , "FONT_CHAR_S_CAPITAL_WITH_CEDILLA"     , { "Ş" } , '\225' }, // Ş
     {  FONT_CHAR_S_SMALL_WITH_CEDILLA       , "FONT_CHAR_S_SMALL_WITH_CEDILLA"       , { "ş" } , '\226' }, // ş
     {  FONT_CHAR_G_CAPITAL_WITH_BREVE       , "FONT_CHAR_G_CAPITAL_WITH_BREVE"       , { "Ğ" } , '\227' }, // Ğ
     {  FONT_CHAR_G_SMALL_WITH_BREVE         , "FONT_CHAR_G_SMALL_WITH_BREVE"         , { "ğ" } , '\230' }, // ğ
     {  FONT_CHAR_CAPITAL_R_WITH_CARON       , "FONT_CHAR_CAPITAL_R_WITH_CARON"       , { "Ř" } , '\231' }, // Ř
     {  FONT_CHAR_SMALL_R_WITH_CARON         , "FONT_CHAR_SMALL_R_WITH_CARON"         , { "ř" } , '\233' }, // ř
     {  FONT_CHAR_CAPITAL_E_WITH_CARON       , "FONT_CHAR_CAPITAL_E_WITH_CARON"       , { "Ě" } , '\234' }, // Ě
     {  FONT_CHAR_SMALL_E_WITH_CARON         , "FONT_CHAR_SMALL_E_WITH_CARON"         , { "ě" } , '\235' }, // ě
     {  FONT_CHAR_CAPITAL_U_WITH_RING_ABOVE  , "FONT_CHAR_CAPITAL_U_WITH_RING_ABOVE"  , { "Ů" } , '\240' }, // Ů
     {  FONT_CHAR_SMALL_U_WITH_RING_ABOVE    , "FONT_CHAR_SMALL_U_WITH_RING_ABOVE"    , { "ů" } , '\241' }, // ů
     {  FONT_CHAR_CAPITAL_O_WITH_DOUBLE_ACUTE, "FONT_CHAR_CAPITAL_O_WITH_DOUBLE_ACUTE", { "Ő" } , '\242' }, // Ő
     {  FONT_CHAR_SMALL_O_WITH_DOUBLE_ACUTE  , "FONT_CHAR_SMALL_O_WITH_DOUBLE_ACUTE"  , { "ő" } , '\243' }, // ő
     {  FONT_CHAR_CAPITAL_U_WITH_DOUBLE_ACUTE, "FONT_CHAR_CAPITAL_U_WITH_DOUBLE_ACUTE", { "Ű" } , '\244' }, // Ű
     {  FONT_CHAR_SMALL_U_WITH_DOUBLE_ACUTE  , "FONT_CHAR_SMALL_U_WITH_DOUBLE_ACUTE"  , { "ű" } , '\245' }  // ű
};
#endif // BUILD_FONT_TOOL || CHAR_CONSTANTS_ONLY

#if ! defined(CHAR_CONSTANTS_ONLY)

#if ! defined(PLATFORM_GD77S)

const uint8_t font_6x8[] = {
   0x00, 0x00, // Ignored.
   32        , // first char ASCII code.
   255       , // last char ASCII code.
   6         , // width of the character in pixels.
   8         , // height of the character in pixels.
   1         , // number of line of bytes per character.
   6         , // bytes per character (normally width * height / 8).
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 32] ' '
   0x00, 0x00, 0x5f, 0x00, 0x00, 0x00, // [ 33] '!'
   0x00, 0x07, 0x00, 0x07, 0x00, 0x00, // [ 34] '"'
   0x14, 0x7f, 0x14, 0x7f, 0x14, 0x00, // [ 35] '#'
   0x24, 0x2a, 0x7f, 0x2a, 0x12, 0x00, // [ 36] '$'
   0x23, 0x13, 0x08, 0x64, 0x62, 0x00, // [ 37] '%'
   0x36, 0x49, 0x55, 0x22, 0x50, 0x00, // [ 38] '&'
   0x00, 0x05, 0x03, 0x00, 0x00, 0x00, // [ 39] '''
   0x00, 0x1c, 0x22, 0x41, 0x00, 0x00, // [ 40] '('
   0x00, 0x41, 0x22, 0x1c, 0x00, 0x00, // [ 41] ')'
   0x14, 0x08, 0x3e, 0x08, 0x14, 0x00, // [ 42] '*'
   0x08, 0x08, 0x3e, 0x08, 0x08, 0x00, // [ 43] '+'
   0x00, 0x50, 0x30, 0x00, 0x00, 0x00, // [ 44] ','
   0x08, 0x08, 0x08, 0x08, 0x08, 0x00, // [ 45] '-'
   0x00, 0x60, 0x60, 0x00, 0x00, 0x00, // [ 46] '.'
   0x20, 0x10, 0x08, 0x04, 0x02, 0x00, // [ 47] '/'
   0x3e, 0x51, 0x49, 0x45, 0x3e, 0x00, // [ 48] '0'
   0x00, 0x42, 0x7f, 0x40, 0x00, 0x00, // [ 49] '1'
   0x42, 0x61, 0x51, 0x49, 0x46, 0x00, // [ 50] '2'
   0x21, 0x41, 0x45, 0x4b, 0x31, 0x00, // [ 51] '3'
   0x18, 0x14, 0x12, 0x7f, 0x10, 0x00, // [ 52] '4'
   0x27, 0x45, 0x45, 0x45, 0x39, 0x00, // [ 53] '5'
   0x3c, 0x4a, 0x49, 0x49, 0x30, 0x00, // [ 54] '6'
   0x01, 0x71, 0x09, 0x05, 0x03, 0x00, // [ 55] '7'
   0x36, 0x49, 0x49, 0x49, 0x36, 0x00, // [ 56] '8'
   0x06, 0x49, 0x49, 0x29, 0x1e, 0x00, // [ 57] '9'
   0x00, 0x36, 0x36, 0x00, 0x00, 0x00, // [ 58] ':'
   0x00, 0x56, 0x36, 0x00, 0x00, 0x00, // [ 59] ';'
   0x08, 0x14, 0x22, 0x41, 0x00, 0x00, // [ 60] '<'
   0x14, 0x14, 0x14, 0x14, 0x14, 0x00, // [ 61] '='
   0x00, 0x41, 0x22, 0x14, 0x08, 0x00, // [ 62] '>'
   0x02, 0x01, 0x51, 0x09, 0x06, 0x00, // [ 63] '?'
   0x3e, 0x41, 0x4f, 0x49, 0x26, 0x00, // [ 64] '@'
   0x7e, 0x11, 0x11, 0x11, 0x7e, 0x00, // [ 65] 'A'
   0x7f, 0x49, 0x49, 0x49, 0x36, 0x00, // [ 66] 'B'
   0x3e, 0x41, 0x41, 0x41, 0x22, 0x00, // [ 67] 'C'
   0x7f, 0x41, 0x41, 0x22, 0x1c, 0x00, // [ 68] 'D'
   0x7f, 0x49, 0x49, 0x49, 0x41, 0x00, // [ 69] 'E'
   0x7f, 0x09, 0x09, 0x09, 0x01, 0x00, // [ 70] 'F'
   0x3e, 0x41, 0x49, 0x49, 0x7a, 0x00, // [ 71] 'G'
   0x7f, 0x08, 0x08, 0x08, 0x7f, 0x00, // [ 72] 'H'
   0x00, 0x41, 0x7f, 0x41, 0x00, 0x00, // [ 73] 'I'
   0x20, 0x40, 0x41, 0x3f, 0x01, 0x00, // [ 74] 'J'
   0x7f, 0x08, 0x14, 0x22, 0x41, 0x00, // [ 75] 'K'
   0x7f, 0x40, 0x40, 0x40, 0x40, 0x00, // [ 76] 'L'
   0x7f, 0x02, 0x0c, 0x02, 0x7f, 0x00, // [ 77] 'M'
   0x7f, 0x04, 0x08, 0x10, 0x7f, 0x00, // [ 78] 'N'
   0x3e, 0x41, 0x41, 0x41, 0x3e, 0x00, // [ 79] 'O'
   0x7f, 0x09, 0x09, 0x09, 0x06, 0x00, // [ 80] 'P'
   0x3e, 0x41, 0x51, 0x21, 0x5e, 0x00, // [ 81] 'Q'
   0x7f, 0x09, 0x19, 0x29, 0x46, 0x00, // [ 82] 'R'
   0x46, 0x49, 0x49, 0x49, 0x31, 0x00, // [ 83] 'S'
   0x01, 0x01, 0x7f, 0x01, 0x01, 0x00, // [ 84] 'T'
   0x3f, 0x40, 0x40, 0x40, 0x3f, 0x00, // [ 85] 'U'
   0x03, 0x1c, 0x60, 0x1c, 0x03, 0x00, // [ 86] 'V'
   0x3f, 0x40, 0x38, 0x40, 0x3f, 0x00, // [ 87] 'W'
   0x63, 0x14, 0x08, 0x14, 0x63, 0x00, // [ 88] 'X'
   0x07, 0x08, 0x70, 0x08, 0x07, 0x00, // [ 89] 'Y'
   0x61, 0x51, 0x49, 0x45, 0x43, 0x00, // [ 90] 'Z'
   0x00, 0x7f, 0x41, 0x41, 0x00, 0x00, // [ 91] '['
   0x02, 0x04, 0x08, 0x10, 0x20, 0x00, // [ 92] '\'
   0x00, 0x41, 0x41, 0x7f, 0x00, 0x00, // [ 93] ']'
   0x04, 0x02, 0x01, 0x02, 0x04, 0x00, // [ 94] '^'
   0x40, 0x40, 0x40, 0x40, 0x40, 0x00, // [ 95] '_'
   0x00, 0x01, 0x02, 0x04, 0x00, 0x00, // [ 96] '`'
   0x20, 0x54, 0x54, 0x54, 0x78, 0x00, // [ 97] 'a'
   0x7f, 0x48, 0x44, 0x44, 0x38, 0x00, // [ 98] 'b'
   0x38, 0x44, 0x44, 0x44, 0x20, 0x00, // [ 99] 'c'
   0x38, 0x44, 0x44, 0x48, 0x7f, 0x00, // [100] 'd'
   0x38, 0x54, 0x54, 0x54, 0x18, 0x00, // [101] 'e'
   0x08, 0x7e, 0x09, 0x01, 0x02, 0x00, // [102] 'f'
   0x0c, 0x52, 0x52, 0x52, 0x3e, 0x00, // [103] 'g'
   0x7f, 0x08, 0x04, 0x04, 0x78, 0x00, // [104] 'h'
   0x00, 0x44, 0x7d, 0x40, 0x00, 0x00, // [105] 'i'
   0x20, 0x40, 0x44, 0x3d, 0x00, 0x00, // [106] 'j'
   0x7f, 0x10, 0x28, 0x44, 0x00, 0x00, // [107] 'k'
   0x00, 0x41, 0x7f, 0x40, 0x00, 0x00, // [108] 'l'
   0x7c, 0x04, 0x18, 0x04, 0x78, 0x00, // [109] 'm'
   0x7c, 0x08, 0x04, 0x04, 0x78, 0x00, // [110] 'n'
   0x38, 0x44, 0x44, 0x44, 0x38, 0x00, // [111] 'o'
   0x7c, 0x14, 0x14, 0x14, 0x08, 0x00, // [112] 'p'
   0x08, 0x14, 0x14, 0x18, 0x7c, 0x00, // [113] 'q'
   0x7c, 0x08, 0x04, 0x04, 0x08, 0x00, // [114] 'r'
   0x48, 0x54, 0x54, 0x54, 0x20, 0x00, // [115] 's'
   0x04, 0x3f, 0x44, 0x40, 0x20, 0x00, // [116] 't'
   0x3c, 0x40, 0x40, 0x20, 0x7c, 0x00, // [117] 'u'
   0x1c, 0x20, 0x40, 0x20, 0x1c, 0x00, // [118] 'v'
   0x3c, 0x40, 0x30, 0x40, 0x3c, 0x00, // [119] 'w'
   0x44, 0x28, 0x10, 0x28, 0x44, 0x00, // [120] 'x'
   0x0c, 0x50, 0x50, 0x50, 0x3c, 0x00, // [121] 'y'
   0x44, 0x64, 0x54, 0x4c, 0x44, 0x00, // [122] 'z'
   0x00, 0x08, 0x36, 0x41, 0x00, 0x00, // [123] '{'
   0x00, 0x00, 0x7f, 0x00, 0x00, 0x00, // [124] '|'
   0x00, 0x41, 0x36, 0x08, 0x00, 0x00, // [125] '}'
   0x10, 0x08, 0x08, 0x10, 0x08, 0x00, // [126] '~'
   0x38, 0x45, 0x46, 0x45, 0x28, 0x00, // [127] '*Č'
   0x38, 0x45, 0x46, 0x45, 0x20, 0x00, // [128] '*č'
   0x38, 0x44, 0x46, 0x45, 0x29, 0x00, // [129] '*Ć'
   0x39, 0x45, 0x46, 0x44, 0x20, 0x00, // [130] '*ć'
   0x3e, 0x09, 0x09, 0xc9, 0xbe, 0x80, // [131] '*Ą'
   0x10, 0x2a, 0x2a, 0xea, 0xbc, 0x80, // [132] '*ą'
   0x3f, 0x25, 0x25, 0xe5, 0xa1, 0x80, // [133] '*Ę'
   0x1c, 0x2a, 0x2a, 0xea, 0x8c, 0x80, // [134] '*ę'
   0x10, 0x7f, 0x48, 0x44, 0x40, 0x00, // [135] '*Ł'
   0x00, 0x51, 0x7f, 0x48, 0x00, 0x00, // [136] '*ł'
   0x7c, 0x08, 0x12, 0x21, 0x7d, 0x00, // [137] '*Ń'
   0x48, 0x55, 0x56, 0x55, 0x24, 0x00, // [138] '*Š'
   0x7c, 0x0a, 0x05, 0x05, 0x78, 0x00, // [139] '*ń'
   0x48, 0x54, 0x56, 0x55, 0x25, 0x00, // [140] '*Ś'
   0x48, 0x54, 0x56, 0x55, 0x21, 0x00, // [141] '*ś'
   0x64, 0x65, 0x56, 0x4d, 0x4c, 0x00, // [142] '*Ž'
   0x64, 0x64, 0x56, 0x4d, 0x4d, 0x00, // [143] '*Ź'
   0x44, 0x64, 0x56, 0x4d, 0x45, 0x00, // [144] '*ź'
   0x64, 0x64, 0x55, 0x4c, 0x4c, 0x00, // [145] '*Ż'
   0x44, 0x64, 0x55, 0x4c, 0x44, 0x00, // [146] '*ż'
   0x00, 0x44, 0x7c, 0x40, 0x00, 0x00, // [147] '*ı'
   0x00, 0x44, 0x7d, 0x44, 0x00, 0x00, // [148] '*İ'
   0x22, 0xa5, 0x65, 0x25, 0x19, 0x00, // [149] '*Ş'
   0x24, 0xaa, 0x6a, 0x2a, 0x10, 0x00, // [150] '*ş'
   0x3c, 0x45, 0x56, 0x55, 0x74, 0x00, // [151] '*Ğ'
   0x08, 0x55, 0x56, 0x55, 0x3c, 0x00, // [152] '*ğ'
   0x7c, 0x15, 0x16, 0x35, 0x48, 0x00, // [153] '*Ř'
   0x48, 0x55, 0x56, 0x55, 0x20, 0x00, // [154] '*š'
   0x7c, 0x09, 0x06, 0x05, 0x08, 0x00, // [155] '*ř'
   0x7c, 0x55, 0x56, 0x55, 0x44, 0x00, // [156] '*Ě'
   0x38, 0x55, 0x56, 0x55, 0x18, 0x00, // [157] '*ě'
   0x44, 0x65, 0x56, 0x4d, 0x44, 0x00, // [158] '*ž'
   0x0c, 0x11, 0x60, 0x11, 0x0c, 0x00, // [159] '*Ÿ'
   0x3c, 0x42, 0x45, 0x42, 0x3c, 0x00, // [160] '*Ů'
   0x3c, 0x42, 0x45, 0x22, 0x7c, 0x00, // [161] '*ů'
   0x38, 0x46, 0x45, 0x46, 0x39, 0x00, // [162] '*Ő'
   0x38, 0x46, 0x45, 0x46, 0x39, 0x00, // [163] '*ő'
   0x3c, 0x42, 0x41, 0x42, 0x3d, 0x00, // [164] '*Ű'
   0x3c, 0x42, 0x41, 0x22, 0x7d, 0x00, // [165] '*ű'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [166] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [167] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [168] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [169] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [170] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [171] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [172] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [173] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [174] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [175] 'FREE'
   0x00, 0x06, 0x09, 0x09, 0x06, 0x00, // [176] '°'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [177] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [178] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [179] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [180] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [181] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [182] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [183] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [184] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [185] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [186] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [187] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [188] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [189] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [190] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [191] 'FREE'
   0x79, 0x25, 0x26, 0x24, 0x78, 0x00, // [192] 'À'
   0x78, 0x24, 0x26, 0x25, 0x79, 0x00, // [193] 'Á'
   0x7a, 0x25, 0x25, 0x25, 0x7a, 0x00, // [194] 'Â'
   0x7a, 0x25, 0x25, 0x26, 0x79, 0x00, // [195] 'Ã'
   0x78, 0x25, 0x24, 0x25, 0x78, 0x00, // [196] 'Ä'
   0x78, 0x26, 0x25, 0x26, 0x78, 0x00, // [197] 'Å'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [198] 'FREE'
   0x1e, 0xa1, 0x61, 0x21, 0x12, 0x00, // [199] 'Ç'
   0x7c, 0x55, 0x55, 0x56, 0x44, 0x00, // [200] 'È'
   0x7c, 0x56, 0x55, 0x55, 0x44, 0x00, // [201] 'É'
   0x7e, 0x55, 0x55, 0x55, 0x46, 0x00, // [202] 'Ê'
   0x7c, 0x55, 0x54, 0x55, 0x44, 0x00, // [203] 'Ë'
   0x01, 0x45, 0x7e, 0x44, 0x00, 0x00, // [204] 'Ì'
   0x00, 0x44, 0x7e, 0x45, 0x01, 0x00, // [205] 'Í'
   0x02, 0x45, 0x7d, 0x45, 0x02, 0x00, // [206] 'Î'
   0x00, 0x45, 0x7c, 0x45, 0x00, 0x00, // [207] 'Ï'
   0x7f, 0x49, 0x49, 0x22, 0x1c, 0x00, // [208] 'Ð'
   0x7e, 0x09, 0x11, 0x22, 0x7d, 0x00, // [209] 'Ñ'
   0x39, 0x45, 0x46, 0x44, 0x38, 0x00, // [210] 'Ò'
   0x38, 0x44, 0x46, 0x45, 0x39, 0x00, // [211] 'Ó'
   0x3a, 0x45, 0x45, 0x45, 0x3a, 0x00, // [212] 'Ô'
   0x3a, 0x45, 0x45, 0x46, 0x39, 0x00, // [213] 'Õ'
   0x38, 0x45, 0x44, 0x45, 0x38, 0x00, // [214] 'Ö'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [215] 'FREE'
   0xb8, 0x64, 0x54, 0x4c, 0x3a, 0x00, // [216] 'Ø'
   0x3d, 0x41, 0x42, 0x40, 0x3c, 0x00, // [217] 'Ù'
   0x3c, 0x40, 0x42, 0x41, 0x3d, 0x00, // [218] 'Ú'
   0x3c, 0x42, 0x41, 0x42, 0x3c, 0x00, // [219] 'Û'
   0x3c, 0x41, 0x40, 0x41, 0x3c, 0x00, // [220] 'Ü'
   0x0c, 0x10, 0x62, 0x11, 0x0d, 0x00, // [221] 'Ý'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [222] 'FREE'
   0xff, 0x01, 0x45, 0x25, 0x1a, 0x00, // [223] 'ß'
   0x21, 0x55, 0x56, 0x54, 0x78, 0x00, // [224] 'à'
   0x20, 0x54, 0x56, 0x55, 0x79, 0x00, // [225] 'á'
   0x22, 0x55, 0x55, 0x55, 0x7a, 0x00, // [226] 'â'
   0x22, 0x55, 0x55, 0x56, 0x79, 0x00, // [227] 'ã'
   0x20, 0x55, 0x54, 0x55, 0x78, 0x00, // [228] 'ä'
   0x20, 0x56, 0x55, 0x56, 0x78, 0x00, // [229] 'å'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [230] 'FREE'
   0x18, 0xa4, 0x64, 0x24, 0x10, 0x00, // [231] 'ç'
   0x39, 0x55, 0x56, 0x54, 0x18, 0x00, // [232] 'è'
   0x38, 0x54, 0x56, 0x55, 0x19, 0x00, // [233] 'é'
   0x3a, 0x55, 0x55, 0x55, 0x1a, 0x00, // [234] 'ê'
   0x38, 0x55, 0x54, 0x55, 0x18, 0x00, // [235] 'ë'
   0x01, 0x45, 0x7e, 0x40, 0x00, 0x00, // [236] 'ì'
   0x00, 0x44, 0x7e, 0x41, 0x01, 0x00, // [237] 'í'
   0x02, 0x45, 0x7d, 0x41, 0x02, 0x00, // [238] 'î'
   0x00, 0x45, 0x7c, 0x41, 0x00, 0x00, // [239] 'ï'
   0x30, 0x48, 0x48, 0x52, 0x7f, 0x02, // [240] 'ð'
   0x7e, 0x09, 0x05, 0x06, 0x79, 0x00, // [241] 'ñ'
   0x39, 0x45, 0x46, 0x44, 0x38, 0x00, // [242] 'ò'
   0x38, 0x44, 0x46, 0x45, 0x39, 0x00, // [243] 'ó'
   0x3a, 0x45, 0x45, 0x45, 0x3a, 0x00, // [244] 'ô'
   0x3a, 0x45, 0x45, 0x46, 0x39, 0x00, // [245] 'õ'
   0x38, 0x45, 0x44, 0x45, 0x38, 0x00, // [246] 'ö'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [247] 'FREE'
   0xb8, 0x64, 0x54, 0x4c, 0x3a, 0x00, // [248] 'ø'
   0x3d, 0x41, 0x42, 0x20, 0x7c, 0x00, // [249] 'ù'
   0x3c, 0x40, 0x42, 0x21, 0x7d, 0x00, // [250] 'ú'
   0x3c, 0x42, 0x41, 0x22, 0x7c, 0x00, // [251] 'û'
   0x3c, 0x41, 0x40, 0x21, 0x7c, 0x00, // [252] 'ü'
   0x0c, 0x50, 0x52, 0x51, 0x3d, 0x00, // [253] 'ý'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [254] 'FREE'
   0x0c, 0x51, 0x50, 0x51, 0x3c, 0x00  // [255] 'ÿ'
};


// 5x7 font (in 6x8 cell)
const uint8_t font_6x8_bold[] = {
   0x00, 0x00, // Ignored.
   32        , // first char ASCII code.
   255       , // last char ASCII code.
   6         , // width of the character in pixels.
   8         , // height of the character in pixels.
   1         , // number of line of bytes per character.
   6         , // bytes per character (normally width * height / 8).
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 32] ' '
   0x00, 0x5f, 0x5f, 0x00, 0x00, 0x00, // [ 33] '!'
   0x03, 0x07, 0x00, 0x03, 0x07, 0x00, // [ 34] '"'
   0x14, 0x7f, 0x14, 0x7f, 0x14, 0x00, // [ 35] '#'
   0x2e, 0x7a, 0x7f, 0x2f, 0x3a, 0x00, // [ 36] '$'
   0x67, 0x37, 0x08, 0x76, 0x73, 0x00, // [ 37] '%'
   0x76, 0x4f, 0x59, 0x27, 0x56, 0x00, // [ 38] '&'
   0x05, 0x07, 0x03, 0x00, 0x00, 0x00, // [ 39] '''
   0x1c, 0x3e, 0x63, 0x41, 0x00, 0x00, // [ 40] '('
   0x41, 0x63, 0x3e, 0x1c, 0x00, 0x00, // [ 41] ')'
   0x14, 0x08, 0x3e, 0x08, 0x14, 0x00, // [ 42] '*'
   0x08, 0x08, 0x3e, 0x08, 0x08, 0x00, // [ 43] '+'
   0x50, 0x70, 0x30, 0x00, 0x00, 0x00, // [ 44] ','
   0x08, 0x08, 0x08, 0x08, 0x08, 0x00, // [ 45] '-'
   0x60, 0x60, 0x60, 0x00, 0x00, 0x00, // [ 46] '.'
   0x60, 0x30, 0x18, 0x0c, 0x06, 0x00, // [ 47] '/'
   0x3e, 0x7b, 0x49, 0x67, 0x3e, 0x00, // [ 48] '0'
   0x42, 0x7f, 0x7f, 0x40, 0x00, 0x00, // [ 49] '1'
   0x63, 0x73, 0x59, 0x4f, 0x46, 0x00, // [ 50] '2'
   0x63, 0x4b, 0x49, 0x7f, 0x36, 0x00, // [ 51] '3'
   0x1c, 0x1e, 0x73, 0x7f, 0x10, 0x00, // [ 52] '4'
   0x67, 0x67, 0x45, 0x7d, 0x39, 0x00, // [ 53] '5'
   0x3c, 0x7e, 0x4b, 0x7b, 0x30, 0x00, // [ 54] '6'
   0x01, 0x79, 0x7d, 0x07, 0x03, 0x00, // [ 55] '7'
   0x36, 0x7f, 0x49, 0x7f, 0x36, 0x00, // [ 56] '8'
   0x06, 0x4f, 0x69, 0x3f, 0x1e, 0x00, // [ 57] '9'
   0x36, 0x36, 0x36, 0x00, 0x00, 0x00, // [ 58] ':'
   0x56, 0x76, 0x36, 0x00, 0x00, 0x00, // [ 59] ';'
   0x08, 0x1c, 0x36, 0x63, 0x41, 0x00, // [ 60] '<'
   0x14, 0x14, 0x14, 0x14, 0x14, 0x00, // [ 61] '='
   0x41, 0x63, 0x36, 0x1c, 0x08, 0x00, // [ 62] '>'
   0x03, 0x53, 0x59, 0x0f, 0x06, 0x00, // [ 63] '?'
   0x7e, 0x63, 0x4f, 0x6b, 0x6f, 0x00, // [ 64] '@'
   0x7e, 0x7f, 0x13, 0x7f, 0x7e, 0x00, // [ 65] 'A'
   0x7f, 0x6b, 0x6b, 0x7f, 0x36, 0x00, // [ 66] 'B'
   0x3e, 0x7f, 0x63, 0x63, 0x36, 0x00, // [ 67] 'C'
   0x7f, 0x7f, 0x63, 0x7f, 0x3e, 0x00, // [ 68] 'D'
   0x7f, 0x7f, 0x6b, 0x6b, 0x63, 0x00, // [ 69] 'E'
   0x7f, 0x7f, 0x0b, 0x0b, 0x03, 0x00, // [ 70] 'F'
   0x3e, 0x7f, 0x63, 0x6b, 0x7b, 0x00, // [ 71] 'G'
   0x7f, 0x7f, 0x08, 0x7f, 0x7f, 0x00, // [ 72] 'H'
   0x00, 0x41, 0x7f, 0x7f, 0x41, 0x00, // [ 73] 'I'
   0x70, 0x61, 0x7f, 0x3f, 0x01, 0x00, // [ 74] 'J'
   0x7f, 0x7f, 0x1c, 0x77, 0x63, 0x00, // [ 75] 'K'
   0x7f, 0x7f, 0x60, 0x60, 0x60, 0x00, // [ 76] 'L'
   0x7f, 0x76, 0x0c, 0x76, 0x7f, 0x00, // [ 77] 'M'
   0x7f, 0x6f, 0x1c, 0x7b, 0x7f, 0x00, // [ 78] 'N'
   0x3e, 0x7f, 0x41, 0x7f, 0x3e, 0x00, // [ 79] 'O'
   0x7f, 0x7f, 0x09, 0x0f, 0x06, 0x00, // [ 80] 'P'
   0x3e, 0x6f, 0x71, 0x7f, 0x5e, 0x00, // [ 81] 'Q'
   0x7f, 0x7f, 0x09, 0x7f, 0x76, 0x00, // [ 82] 'R'
   0x6f, 0x6f, 0x49, 0x7b, 0x7b, 0x00, // [ 83] 'S'
   0x01, 0x7f, 0x7f, 0x01, 0x00, 0x00, // [ 84] 'T'
   0x3f, 0x7f, 0x40, 0x7f, 0x1f, 0x00, // [ 85] 'U'
   0x07, 0x1f, 0x60, 0x1f, 0x07, 0x00, // [ 86] 'V'
   0x3f, 0x67, 0x38, 0x67, 0x3f, 0x00, // [ 87] 'W'
   0x63, 0x77, 0x08, 0x77, 0x63, 0x00, // [ 88] 'X'
   0x03, 0x0f, 0x70, 0x0f, 0x03, 0x00, // [ 89] 'Y'
   0x71, 0x79, 0x5d, 0x4f, 0x47, 0x00, // [ 90] 'Z'
   0x7f, 0x7f, 0x41, 0x41, 0x00, 0x00, // [ 91] '['
   0x06, 0x0c, 0x18, 0x30, 0x60, 0x00, // [ 92] '\'
   0x41, 0x41, 0x7f, 0x7f, 0x00, 0x00, // [ 93] ']'
   0x0c, 0x06, 0x03, 0x06, 0x0c, 0x00, // [ 94] '^'
   0x60, 0x60, 0x60, 0x60, 0x60, 0x00, // [ 95] '_'
   0x01, 0x03, 0x06, 0x04, 0x00, 0x00, // [ 96] '`'
   0x70, 0x54, 0x54, 0x7c, 0x78, 0x00, // [ 97] 'a'
   0x7f, 0x7f, 0x44, 0x7c, 0x38, 0x00, // [ 98] 'b'
   0x38, 0x7c, 0x6c, 0x44, 0x6c, 0x00, // [ 99] 'c'
   0x38, 0x7c, 0x44, 0x7f, 0x7f, 0x00, // [100] 'd'
   0x38, 0x7c, 0x74, 0x5c, 0x58, 0x00, // [101] 'e'
   0x08, 0x7e, 0x7f, 0x09, 0x01, 0x00, // [102] 'f'
   0x1c, 0x5e, 0x52, 0x7e, 0x3e, 0x00, // [103] 'g'
   0x7f, 0x7f, 0x0c, 0x7c, 0x78, 0x00, // [104] 'h'
   0x44, 0x7d, 0x7d, 0x40, 0x00, 0x00, // [105] 'i'
   0x70, 0x40, 0x7d, 0x7d, 0x00, 0x00, // [106] 'j'
   0x7f, 0x7f, 0x18, 0x1e, 0x72, 0x00, // [107] 'k'
   0x41, 0x7f, 0x7f, 0x40, 0x00, 0x00, // [108] 'l'
   0x7c, 0x6c, 0x18, 0x6c, 0x7c, 0x00, // [109] 'm'
   0x7c, 0x7c, 0x04, 0x7c, 0x78, 0x00, // [110] 'n'
   0x38, 0x7c, 0x44, 0x7c, 0x38, 0x00, // [111] 'o'
   0x7c, 0x7c, 0x14, 0x14, 0x1c, 0x00, // [112] 'p'
   0x1c, 0x14, 0x14, 0x7c, 0x7c, 0x00, // [113] 'q'
   0x7c, 0x7c, 0x04, 0x0c, 0x0c, 0x00, // [114] 'r'
   0x5c, 0x5c, 0x54, 0x74, 0x70, 0x00, // [115] 's'
   0x04, 0x7f, 0x7f, 0x44, 0x20, 0x00, // [116] 't'
   0x7c, 0x7c, 0x40, 0x3c, 0x7c, 0x00, // [117] 'u'
   0x1c, 0x3c, 0x40, 0x3c, 0x1c, 0x00, // [118] 'v'
   0x3c, 0x6c, 0x30, 0x6c, 0x3c, 0x00, // [119] 'w'
   0x6c, 0x7c, 0x10, 0x7c, 0x6c, 0x00, // [120] 'x'
   0x0c, 0x5c, 0x50, 0x7c, 0x7c, 0x00, // [121] 'y'
   0x64, 0x74, 0x7c, 0x5c, 0x4c, 0x00, // [122] 'z'
   0x00, 0x1c, 0x36, 0x63, 0x00, 0x00, // [123] '{'
   0x00, 0x00, 0x7f, 0x00, 0x00, 0x00, // [124] '|'
   0x00, 0x63, 0x36, 0x1c, 0x00, 0x00, // [125] '}'
   0x10, 0x08, 0x08, 0x10, 0x08, 0x00, // [126] '~'
   0x38, 0x7d, 0x66, 0x65, 0x24, 0x00, // [127] '*Č'
   0x38, 0x7d, 0x6e, 0x45, 0x6c, 0x00, // [128] '*č'
   0x38, 0x7c, 0x66, 0x65, 0x25, 0x00, // [129] '*Ć'
   0x38, 0x7c, 0x6e, 0x45, 0x6d, 0x00, // [130] '*ć'
   0x3e, 0x3f, 0x0b, 0xff, 0xbe, 0x80, // [131] '*Ą'
   0x38, 0x2a, 0x2a, 0xfe, 0xbc, 0x80, // [132] '*ą'
   0x3f, 0x3f, 0x25, 0xe5, 0xa1, 0x80, // [133] '*Ę'
   0x1c, 0x3e, 0x3a, 0xee, 0xac, 0x80, // [134] '*ę'
   0x08, 0x7f, 0x7f, 0x64, 0x62, 0x00, // [135] '*Ł'
   0x49, 0x7f, 0x7f, 0x44, 0x00, 0x00, // [136] '*ł'
   0x7c, 0x5c, 0x3a, 0x75, 0x7d, 0x00, // [137] '*Ń'
   0x5c, 0x5d, 0x56, 0x75, 0x74, 0x00, // [138] '*Š'
   0x7c, 0x7c, 0x06, 0x7d, 0x79, 0x00, // [139] '*ń'
   0x5c, 0x5c, 0x56, 0x75, 0x75, 0x00, // [140] '*Ś'
   0x5c, 0x5c, 0x56, 0x75, 0x71, 0x00, // [141] '*ś'
   0x64, 0x75, 0x7e, 0x5d, 0x4c, 0x00, // [142] '*Ž'
   0x64, 0x74, 0x7e, 0x5d, 0x4d, 0x00, // [143] '*Ź'
   0x64, 0x74, 0x56, 0x5d, 0x4d, 0x00, // [144] '*ź'
   0x64, 0x74, 0x7d, 0x5d, 0x4c, 0x00, // [145] '*Ż'
   0x64, 0x74, 0x55, 0x5d, 0x4c, 0x00, // [146] '*ż'
   0x44, 0x7c, 0x7c, 0x40, 0x00, 0x00, // [147] '*ı'
   0x00, 0x44, 0x7d, 0x7d, 0x44, 0x00, // [148] '*İ'
   0x37, 0xb7, 0x65, 0x3d, 0x3d, 0x00, // [149] '*Ş'
   0x2e, 0xae, 0x6a, 0x3a, 0x38, 0x00, // [150] '*ş'
   0x3c, 0x7f, 0x62, 0x6b, 0x7a, 0x00, // [151] '*Ğ'
   0x1c, 0x5f, 0x52, 0x7f, 0x3e, 0x00, // [152] '*ğ'
   0x7e, 0x7f, 0x0a, 0x7f, 0x74, 0x00, // [153] '*Ř'
   0x5c, 0x5d, 0x56, 0x75, 0x70, 0x00, // [154] '*š'
   0x7c, 0x7d, 0x06, 0x0d, 0x0c, 0x00, // [155] '*ř'
   0x7e, 0x7f, 0x6a, 0x6b, 0x62, 0x00, // [156] '*Ě'
   0x38, 0x7d, 0x76, 0x5d, 0x58, 0x00, // [157] '*ě'
   0x64, 0x75, 0x56, 0x5d, 0x4c, 0x00, // [158] '*ž'
   0x0c, 0x1d, 0x60, 0x1d, 0x0c, 0x00, // [159] '*Ÿ'
   0x3c, 0x7e, 0x45, 0x7e, 0x3c, 0x00, // [160] '*Ů'
   0x7c, 0x7e, 0x45, 0x3e, 0x7c, 0x00, // [161] '*ů'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [162] '*Ő'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [163] '*ő'
   0x3f, 0x7f, 0x40, 0x7f, 0x1f, 0x00, // [164] '*Ű'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [165] '*ű'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [166] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [167] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [168] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [169] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [170] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [171] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [172] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [173] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [174] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [175] 'FREE'
   0x00, 0x06, 0x09, 0x09, 0x06, 0x00, // [176] '°'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [177] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [178] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [179] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [180] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [181] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [182] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [183] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [184] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [185] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [186] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [187] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [188] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [189] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [190] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [191] 'FREE'
   0x79, 0x7d, 0x2e, 0x7c, 0x78, 0x00, // [192] 'À'
   0x78, 0x7c, 0x2e, 0x7d, 0x79, 0x00, // [193] 'Á'
   0x7a, 0x7d, 0x2d, 0x7d, 0x7a, 0x00, // [194] 'Â'
   0x7a, 0x7d, 0x2d, 0x7e, 0x79, 0x00, // [195] 'Ã'
   0x78, 0x7d, 0x2c, 0x7d, 0x78, 0x00, // [196] 'Ä'
   0x78, 0x7e, 0x2d, 0x7e, 0x78, 0x00, // [197] 'Å'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [198] 'FREE'
   0x1e, 0xbf, 0x73, 0x33, 0x1a, 0x00, // [199] 'Ç'
   0x7d, 0x7d, 0x56, 0x54, 0x44, 0x00, // [200] 'È'
   0x7c, 0x7c, 0x56, 0x55, 0x45, 0x00, // [201] 'É'
   0x7c, 0x7e, 0x55, 0x56, 0x44, 0x00, // [202] 'Ê'
   0x7c, 0x7d, 0x54, 0x55, 0x44, 0x00, // [203] 'Ë'
   0x00, 0x45, 0x7d, 0x7e, 0x44, 0x00, // [204] 'Ì'
   0x00, 0x44, 0x7e, 0x7d, 0x45, 0x00, // [205] 'Í'
   0x02, 0x45, 0x7d, 0x7d, 0x45, 0x02, // [206] 'Î'
   0x00, 0x45, 0x7c, 0x7c, 0x45, 0x00, // [207] 'Ï'
   0x7f, 0x7f, 0x6b, 0x63, 0x3e, 0x00, // [208] 'Ð'
   0x7e, 0x5d, 0x39, 0x76, 0x7d, 0x00, // [209] 'Ñ'
   0x39, 0x7d, 0x46, 0x7c, 0x38, 0x00, // [210] 'Ò'
   0x38, 0x7c, 0x46, 0x7d, 0x39, 0x00, // [211] 'Ó'
   0x3a, 0x7d, 0x45, 0x7d, 0x3a, 0x00, // [212] 'Ô'
   0x3a, 0x7d, 0x45, 0x7e, 0x39, 0x00, // [213] 'Õ'
   0x38, 0x7d, 0x44, 0x7d, 0x38, 0x00, // [214] 'Ö'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [215] 'FREE'
   0xb8, 0x64, 0x54, 0x4c, 0x3a, 0x00, // [216] 'Ø'
   0x3d, 0x7d, 0x42, 0x7c, 0x3c, 0x00, // [217] 'Ù'
   0x3c, 0x7c, 0x42, 0x7d, 0x3d, 0x00, // [218] 'Ú'
   0x3c, 0x7e, 0x41, 0x7e, 0x3c, 0x00, // [219] 'Û'
   0x3c, 0x7d, 0x40, 0x7d, 0x3c, 0x00, // [220] 'Ü'
   0x0c, 0x1c, 0x62, 0x1d, 0x0d, 0x00, // [221] 'Ý'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [222] 'FREE'
   0xff, 0x03, 0x8b, 0x7f, 0x36, 0x00, // [223] 'ß'
   0x71, 0x55, 0x56, 0x7c, 0x78, 0x00, // [224] 'à'
   0x70, 0x54, 0x56, 0x7d, 0x79, 0x00, // [225] 'á'
   0x72, 0x55, 0x55, 0x7d, 0x7a, 0x00, // [226] 'â'
   0x72, 0x55, 0x55, 0x7e, 0x79, 0x00, // [227] 'ã'
   0x70, 0x55, 0x54, 0x7d, 0x78, 0x00, // [228] 'ä'
   0x70, 0x56, 0x55, 0x7e, 0x78, 0x00, // [229] 'å'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [230] 'FREE'
   0x18, 0x3c, 0xa4, 0x64, 0x24, 0x00, // [231] 'ç'
   0x39, 0x7d, 0x76, 0x5c, 0x58, 0x00, // [232] 'è'
   0x38, 0x7c, 0x76, 0x5d, 0x59, 0x00, // [233] 'é'
   0x3a, 0x7d, 0x75, 0x5d, 0x5a, 0x00, // [234] 'ê'
   0x38, 0x7d, 0x74, 0x5d, 0x58, 0x00, // [235] 'ë'
   0x45, 0x7d, 0x7e, 0x40, 0x00, 0x00, // [236] 'ì'
   0x44, 0x7e, 0x7d, 0x41, 0x00, 0x00, // [237] 'í'
   0x46, 0x7d, 0x7d, 0x42, 0x00, 0x00, // [238] 'î'
   0x45, 0x7c, 0x7d, 0x40, 0x00, 0x00, // [239] 'ï'
   0x30, 0x78, 0x4a, 0x7f, 0x7f, 0x02, // [240] 'ð'
   0x7e, 0x7d, 0x05, 0x7e, 0x79, 0x00, // [241] 'ñ'
   0x39, 0x7d, 0x46, 0x7c, 0x38, 0x00, // [242] 'ò'
   0x38, 0x7c, 0x46, 0x7d, 0x39, 0x00, // [243] 'ó'
   0x3a, 0x7d, 0x45, 0x7d, 0x3a, 0x00, // [244] 'ô'
   0x3a, 0x7d, 0x45, 0x7e, 0x39, 0x00, // [245] 'õ'
   0x38, 0x7d, 0x44, 0x7d, 0x38, 0x00, // [246] 'ö'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [247] 'FREE'
   0xb8, 0x64, 0x54, 0x4c, 0x3a, 0x00, // [248] 'ø'
   0x7d, 0x7d, 0x42, 0x3c, 0x7c, 0x00, // [249] 'ù'
   0x7c, 0x7c, 0x42, 0x3d, 0x7d, 0x00, // [250] 'ú'
   0x7c, 0x7e, 0x41, 0x3e, 0x7c, 0x00, // [251] 'û'
   0x7c, 0x7d, 0x40, 0x3d, 0x7c, 0x00, // [252] 'ü'
   0x0c, 0x5c, 0x52, 0x7d, 0x7d, 0x00, // [253] 'ý'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [254] 'FREE'
   0x0c, 0x5d, 0x50, 0x7d, 0x7c, 0x00  // [255] 'ÿ'
};


const uint8_t font_8x8[] = {
   0x00, 0x00, // Ignored.
   32        , // first char ASCII code.
   255       , // last char ASCII code.
   8         , // width of the character in pixels.
   8         , // height of the character in pixels.
   1         , // number of line of bytes per character.
   8         , // bytes per character (normally width * height / 8).
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 32] ' '
   0x00, 0x00, 0x06, 0x5f, 0x5f, 0x06, 0x00, 0x00, // [ 33] '!'
   0x00, 0x07, 0x07, 0x00, 0x07, 0x07, 0x00, 0x00, // [ 34] '"'
   0x14, 0x7f, 0x7f, 0x14, 0x7f, 0x7f, 0x14, 0x00, // [ 35] '#'
   0x24, 0x2e, 0x2a, 0x6b, 0x6b, 0x3a, 0x12, 0x00, // [ 36] '$'
   0x46, 0x66, 0x30, 0x18, 0x0c, 0x66, 0x62, 0x00, // [ 37] '%'
   0x30, 0x7a, 0x4f, 0x5d, 0x37, 0x7a, 0x48, 0x00, // [ 38] '&'
   0x00, 0x04, 0x07, 0x03, 0x00, 0x00, 0x00, 0x00, // [ 39] '''
   0x00, 0x1c, 0x3e, 0x63, 0x41, 0x00, 0x00, 0x00, // [ 40] '('
   0x00, 0x41, 0x63, 0x3e, 0x1c, 0x00, 0x00, 0x00, // [ 41] ')'
   0x08, 0x2a, 0x3e, 0x1c, 0x1c, 0x3e, 0x2a, 0x08, // [ 42] '*'
   0x00, 0x08, 0x08, 0x3e, 0x3e, 0x08, 0x08, 0x00, // [ 43] '+'
   0x00, 0x00, 0x80, 0xe0, 0x60, 0x00, 0x00, 0x00, // [ 44] ','
   0x00, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x00, // [ 45] '-'
   0x00, 0x00, 0x00, 0x60, 0x60, 0x00, 0x00, 0x00, // [ 46] '.'
   0x60, 0x30, 0x18, 0x0c, 0x06, 0x03, 0x01, 0x00, // [ 47] '/'
   0x3e, 0x7f, 0x59, 0x4d, 0x47, 0x7f, 0x3e, 0x00, // [ 48] '0'
   0x40, 0x42, 0x7f, 0x7f, 0x40, 0x40, 0x00, 0x00, // [ 49] '1'
   0x62, 0x73, 0x59, 0x49, 0x6f, 0x66, 0x00, 0x00, // [ 50] '2'
   0x22, 0x63, 0x49, 0x49, 0x7f, 0x36, 0x00, 0x00, // [ 51] '3'
   0x18, 0x1c, 0x16, 0x53, 0x7f, 0x7f, 0x50, 0x00, // [ 52] '4'
   0x27, 0x67, 0x45, 0x45, 0x7d, 0x39, 0x00, 0x00, // [ 53] '5'
   0x3c, 0x7e, 0x4b, 0x49, 0x79, 0x30, 0x00, 0x00, // [ 54] '6'
   0x03, 0x03, 0x71, 0x79, 0x0f, 0x07, 0x00, 0x00, // [ 55] '7'
   0x36, 0x7f, 0x49, 0x49, 0x7f, 0x36, 0x00, 0x00, // [ 56] '8'
   0x06, 0x4f, 0x49, 0x69, 0x3f, 0x1e, 0x00, 0x00, // [ 57] '9'
   0x00, 0x00, 0x00, 0x66, 0x66, 0x00, 0x00, 0x00, // [ 58] ':'
   0x00, 0x00, 0x80, 0xe6, 0x66, 0x00, 0x00, 0x00, // [ 59] ';'
   0x08, 0x1c, 0x36, 0x63, 0x41, 0x00, 0x00, 0x00, // [ 60] '<'
   0x00, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x00, // [ 61] '='
   0x00, 0x41, 0x63, 0x36, 0x1c, 0x08, 0x00, 0x00, // [ 62] '>'
   0x00, 0x02, 0x03, 0x59, 0x5d, 0x07, 0x02, 0x00, // [ 63] '?'
   0x3e, 0x7f, 0x41, 0x5d, 0x5d, 0x5f, 0x0e, 0x00, // [ 64] '@'
   0x7c, 0x7e, 0x13, 0x13, 0x7e, 0x7c, 0x00, 0x00, // [ 65] 'A'
   0x41, 0x7f, 0x7f, 0x49, 0x49, 0x7f, 0x36, 0x00, // [ 66] 'B'
   0x1c, 0x3e, 0x63, 0x41, 0x41, 0x63, 0x22, 0x00, // [ 67] 'C'
   0x41, 0x7f, 0x7f, 0x41, 0x63, 0x3e, 0x1c, 0x00, // [ 68] 'D'
   0x41, 0x7f, 0x7f, 0x49, 0x5d, 0x41, 0x63, 0x00, // [ 69] 'E'
   0x41, 0x7f, 0x7f, 0x49, 0x1d, 0x01, 0x03, 0x00, // [ 70] 'F'
   0x1c, 0x3e, 0x63, 0x41, 0x51, 0x33, 0x72, 0x00, // [ 71] 'G'
   0x7f, 0x7f, 0x08, 0x08, 0x7f, 0x7f, 0x00, 0x00, // [ 72] 'H'
   0x00, 0x41, 0x7f, 0x7f, 0x41, 0x00, 0x00, 0x00, // [ 73] 'I'
   0x30, 0x70, 0x40, 0x41, 0x7f, 0x3f, 0x01, 0x00, // [ 74] 'J'
   0x41, 0x7f, 0x7f, 0x08, 0x1c, 0x77, 0x63, 0x00, // [ 75] 'K'
   0x41, 0x7f, 0x7f, 0x41, 0x40, 0x60, 0x70, 0x00, // [ 76] 'L'
   0x7f, 0x7f, 0x0e, 0x1c, 0x0e, 0x7f, 0x7f, 0x00, // [ 77] 'M'
   0x7f, 0x7f, 0x06, 0x0c, 0x18, 0x7f, 0x7f, 0x00, // [ 78] 'N'
   0x1c, 0x3e, 0x63, 0x41, 0x63, 0x3e, 0x1c, 0x00, // [ 79] 'O'
   0x41, 0x7f, 0x7f, 0x49, 0x09, 0x0f, 0x06, 0x00, // [ 80] 'P'
   0x1e, 0x3f, 0x21, 0x31, 0x61, 0x7f, 0x5e, 0x00, // [ 81] 'Q'
   0x41, 0x7f, 0x7f, 0x09, 0x19, 0x7f, 0x66, 0x00, // [ 82] 'R'
   0x26, 0x6f, 0x4d, 0x49, 0x59, 0x73, 0x32, 0x00, // [ 83] 'S'
   0x03, 0x41, 0x7f, 0x7f, 0x41, 0x03, 0x00, 0x00, // [ 84] 'T'
   0x3f, 0x7f, 0x40, 0x40, 0x7f, 0x7f, 0x00, 0x00, // [ 85] 'U'
   0x07, 0x0f, 0x38, 0x60, 0x1f, 0x03, 0x00, 0x00, // [ 86] 'V'
   0x3f, 0x7f, 0x60, 0x30, 0x60, 0x7f, 0x3f, 0x00, // [ 87] 'W'
   0x63, 0x77, 0x1c, 0x08, 0x1c, 0x77, 0x63, 0x00, // [ 88] 'X'
   0x07, 0x4f, 0x78, 0x78, 0x4f, 0x07, 0x00, 0x00, // [ 89] 'Y'
   0x47, 0x63, 0x71, 0x59, 0x4d, 0x67, 0x73, 0x00, // [ 90] 'Z'
   0x00, 0x7f, 0x7f, 0x41, 0x41, 0x00, 0x00, 0x00, // [ 91] '['
   0x01, 0x03, 0x06, 0x0c, 0x18, 0x30, 0x60, 0x00, // [ 92] '\'
   0x00, 0x41, 0x41, 0x7f, 0x7f, 0x00, 0x00, 0x00, // [ 93] ']'
   0x08, 0x0c, 0x06, 0x03, 0x06, 0x0c, 0x08, 0x00, // [ 94] '^'
   0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, // [ 95] '_'
   0x00, 0x00, 0x03, 0x07, 0x04, 0x00, 0x00, 0x00, // [ 96] '`'
   0x20, 0x74, 0x54, 0x54, 0x3c, 0x78, 0x40, 0x00, // [ 97] 'a'
   0x41, 0x7f, 0x3f, 0x48, 0x48, 0x78, 0x30, 0x00, // [ 98] 'b'
   0x38, 0x7c, 0x44, 0x44, 0x6c, 0x28, 0x00, 0x00, // [ 99] 'c'
   0x30, 0x78, 0x48, 0x49, 0x3f, 0x7f, 0x40, 0x00, // [100] 'd'
   0x38, 0x7c, 0x54, 0x54, 0x5c, 0x18, 0x00, 0x00, // [101] 'e'
   0x48, 0x7e, 0x7f, 0x49, 0x03, 0x06, 0x00, 0x00, // [102] 'f'
   0x98, 0xbc, 0xa4, 0xa4, 0xf8, 0x7c, 0x04, 0x00, // [103] 'g'
   0x41, 0x7f, 0x7f, 0x08, 0x04, 0x7c, 0x78, 0x00, // [104] 'h'
   0x00, 0x44, 0x7d, 0x7d, 0x40, 0x00, 0x00, 0x00, // [105] 'i'
   0x60, 0xe0, 0x80, 0x84, 0xfd, 0x7d, 0x00, 0x00, // [106] 'j'
   0x41, 0x7f, 0x7f, 0x10, 0x38, 0x6c, 0x44, 0x00, // [107] 'k'
   0x00, 0x41, 0x7f, 0x7f, 0x40, 0x00, 0x00, 0x00, // [108] 'l'
   0x7c, 0x7c, 0x18, 0x78, 0x1c, 0x7c, 0x78, 0x00, // [109] 'm'
   0x7c, 0x78, 0x04, 0x04, 0x7c, 0x78, 0x00, 0x00, // [110] 'n'
   0x38, 0x7c, 0x44, 0x44, 0x7c, 0x38, 0x00, 0x00, // [111] 'o'
   0x84, 0xfc, 0xf8, 0xa4, 0x24, 0x3c, 0x18, 0x00, // [112] 'p'
   0x18, 0x3c, 0x24, 0xa4, 0xf8, 0xfc, 0x84, 0x00, // [113] 'q'
   0x44, 0x7c, 0x78, 0x4c, 0x04, 0x0c, 0x18, 0x00, // [114] 'r'
   0x48, 0x5c, 0x54, 0x74, 0x64, 0x24, 0x00, 0x00, // [115] 's'
   0x04, 0x04, 0x3e, 0x7f, 0x44, 0x24, 0x00, 0x00, // [116] 't'
   0x3c, 0x7c, 0x40, 0x40, 0x3c, 0x7c, 0x40, 0x00, // [117] 'u'
   0x1c, 0x3c, 0x60, 0x60, 0x3c, 0x1c, 0x00, 0x00, // [118] 'v'
   0x3c, 0x7c, 0x60, 0x30, 0x60, 0x7c, 0x3c, 0x00, // [119] 'w'
   0x44, 0x6c, 0x38, 0x10, 0x38, 0x6c, 0x44, 0x00, // [120] 'x'
   0x9c, 0xbc, 0xa0, 0xa0, 0xfc, 0x7c, 0x00, 0x00, // [121] 'y'
   0x4c, 0x64, 0x74, 0x5c, 0x4c, 0x64, 0x00, 0x00, // [122] 'z'
   0x08, 0x08, 0x3e, 0x77, 0x41, 0x41, 0x00, 0x00, // [123] '{'
   0x00, 0x00, 0x00, 0x77, 0x77, 0x00, 0x00, 0x00, // [124] '|'
   0x41, 0x41, 0x77, 0x3e, 0x08, 0x08, 0x00, 0x00, // [125] '}'
   0x02, 0x03, 0x01, 0x03, 0x02, 0x03, 0x01, 0x00, // [126] '~'
   0x10, 0x38, 0x6d, 0x46, 0x46, 0x6d, 0x28, 0x00, // [127] '*Č'
   0x38, 0x7c, 0x45, 0x46, 0x6d, 0x28, 0x00, 0x00, // [128] '*č'
   0x10, 0x38, 0x6c, 0x46, 0x45, 0x6d, 0x28, 0x00, // [129] '*Ć'
   0x38, 0x7c, 0x46, 0x45, 0x6d, 0x28, 0x00, 0x00, // [130] '*ć'
   0x3c, 0x3e, 0x0b, 0x0b, 0xfe, 0xbc, 0x80, 0x00, // [131] '*Ą'
   0x10, 0x3a, 0x2a, 0x2a, 0xde, 0xbc, 0xa0, 0x00, // [132] '*ą'
   0x21, 0x3f, 0x3f, 0x25, 0xe5, 0xa1, 0xb3, 0x00, // [133] '*Ę'
   0x1c, 0x3e, 0x2a, 0xea, 0xae, 0x8c, 0x00, 0x00, // [134] '*ę'
   0x00, 0x51, 0x7f, 0x7f, 0x49, 0x64, 0x70, 0x00, // [135] '*Ł'
   0x10, 0x49, 0x7f, 0x7f, 0x44, 0x02, 0x00, 0x00, // [136] '*ł'
   0x7c, 0x7c, 0x08, 0x12, 0x31, 0x7d, 0x7c, 0x00, // [137] '*Ń'
   0x28, 0x6c, 0x5d, 0x56, 0x76, 0x6d, 0x28, 0x00, // [138] '*Š'
   0x7c, 0x7a, 0x05, 0x05, 0x7c, 0x78, 0x00, 0x00, // [139] '*ń'
   0x28, 0x6c, 0x5c, 0x56, 0x75, 0x6d, 0x28, 0x00, // [140] '*Ś'
   0x28, 0x4c, 0x5c, 0x76, 0x65, 0x6d, 0x28, 0x00, // [141] '*ś'
   0x6c, 0x64, 0x75, 0x56, 0x5e, 0x4d, 0x6c, 0x00, // [142] '*Ž'
   0x6c, 0x64, 0x74, 0x56, 0x5d, 0x4d, 0x6c, 0x00, // [143] '*Ź'
   0x4c, 0x64, 0x76, 0x5d, 0x4d, 0x64, 0x00, 0x00, // [144] '*ź'
   0x6c, 0x64, 0x74, 0x55, 0x5d, 0x4c, 0x6c, 0x00, // [145] '*Ż'
   0x4c, 0x64, 0x75, 0x5d, 0x4c, 0x64, 0x00, 0x00, // [146] '*ż'
   0x00, 0x44, 0x7c, 0x7c, 0x40, 0x00, 0x00, 0x00, // [147] '*ı'
   0x00, 0x44, 0x7d, 0x7d, 0x44, 0x00, 0x00, 0x00, // [148] '*İ'
   0x16, 0xa7, 0xad, 0x6d, 0x29, 0x3b, 0x1a, 0x00, // [149] '*Ş'
   0xa4, 0xae, 0x6a, 0x3a, 0x32, 0x12, 0x00, 0x00, // [150] '*ş'
   0x18, 0x3c, 0x65, 0x46, 0x55, 0x34, 0x74, 0x00, // [151] '*Ğ'
   0x98, 0xbc, 0xa5, 0xa6, 0xf9, 0x7c, 0x04, 0x00, // [152] '*ğ'
   0x44, 0x7c, 0x7d, 0x16, 0x15, 0x7c, 0x68, 0x00, // [153] '*Ř'
   0x28, 0x4c, 0x5d, 0x76, 0x65, 0x6c, 0x28, 0x00, // [154] '*š'
   0x44, 0x7d, 0x7a, 0x4d, 0x04, 0x0c, 0x18, 0x00, // [155] '*ř'
   0x44, 0x7c, 0x7d, 0x56, 0x55, 0x44, 0x6c, 0x00, // [156] '*Ě'
   0x38, 0x7d, 0x56, 0x55, 0x5c, 0x18, 0x00, 0x00, // [157] '*ě'
   0x4c, 0x64, 0x75, 0x5e, 0x4d, 0x64, 0x00, 0x00, // [158] '*ž'
   0x0c, 0x5d, 0x70, 0x70, 0x5d, 0x0c, 0x00, 0x00, // [159] '*Ÿ'
   0x7c, 0x7e, 0x45, 0x45, 0x7e, 0x7c, 0x00, 0x00, // [160] '*Ů'
   0x3c, 0x7e, 0x45, 0x45, 0x3e, 0x7c, 0x40, 0x00, // [161] '*ů'
   0x38, 0x7c, 0x46, 0x45, 0x7e, 0x39, 0x00, 0x00, // [162] '*Ő'
   0x38, 0x7c, 0x46, 0x45, 0x7e, 0x39, 0x00, 0x00, // [163] '*ő'
   0x3c, 0x7c, 0x42, 0x41, 0x7e, 0x7d, 0x00, 0x00, // [164] '*Ű'
   0x3c, 0x7c, 0x42, 0x41, 0x3e, 0x7d, 0x40, 0x00, // [165] '*ű'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [166] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [167] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [168] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [169] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [170] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [171] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [172] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [173] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [174] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [175] 'FREE'
   0x00, 0x06, 0x09, 0x09, 0x06, 0x00, 0x00, 0x00, // [176] '°'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [177] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [178] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [179] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [180] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [181] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [182] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [183] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [184] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [185] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [186] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [187] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [188] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [189] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [190] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [191] 'FREE'
   0x70, 0x79, 0x2d, 0x2e, 0x78, 0x70, 0x00, 0x00, // [192] 'À'
   0x70, 0x78, 0x2e, 0x2d, 0x79, 0x70, 0x00, 0x00, // [193] 'Á'
   0x72, 0x79, 0x2d, 0x2d, 0x79, 0x72, 0x00, 0x00, // [194] 'Â'
   0x71, 0x7a, 0x2e, 0x2d, 0x79, 0x72, 0x00, 0x00, // [195] 'Ã'
   0x70, 0x79, 0x2c, 0x2c, 0x79, 0x70, 0x00, 0x00, // [196] 'Ä'
   0x70, 0x7a, 0x2d, 0x2d, 0x7a, 0x70, 0x00, 0x00, // [197] 'Å'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [198] 'FREE'
   0x0c, 0x1e, 0xb3, 0xa1, 0x61, 0x33, 0x12, 0x00, // [199] 'Ç'
   0x44, 0x7c, 0x7d, 0x55, 0x56, 0x44, 0x6c, 0x00, // [200] 'È'
   0x44, 0x7c, 0x7c, 0x56, 0x55, 0x45, 0x6c, 0x00, // [201] 'É'
   0x44, 0x7e, 0x7d, 0x55, 0x55, 0x46, 0x6c, 0x00, // [202] 'Ê'
   0x44, 0x7d, 0x7c, 0x54, 0x54, 0x45, 0x6c, 0x00, // [203] 'Ë'
   0x00, 0x45, 0x7d, 0x7e, 0x44, 0x00, 0x00, 0x00, // [204] 'Ì'
   0x00, 0x44, 0x7e, 0x7d, 0x45, 0x00, 0x00, 0x00, // [205] 'Í'
   0x02, 0x45, 0x7d, 0x7d, 0x45, 0x02, 0x00, 0x00, // [206] 'Î'
   0x00, 0x45, 0x7c, 0x7c, 0x45, 0x00, 0x00, 0x00, // [207] 'Ï'
   0x49, 0x7f, 0x7f, 0x49, 0x63, 0x3e, 0x3c, 0x00, // [208] 'Ð'
   0x7e, 0x7d, 0x09, 0x12, 0x32, 0x7d, 0x7d, 0x00, // [209] 'Ñ'
   0x38, 0x7d, 0x45, 0x46, 0x44, 0x7c, 0x38, 0x00, // [210] 'Ò'
   0x38, 0x7c, 0x44, 0x46, 0x45, 0x7d, 0x38, 0x00, // [211] 'Ó'
   0x3a, 0x7d, 0x45, 0x45, 0x45, 0x7d, 0x3a, 0x00, // [212] 'Ô'
   0x3a, 0x7d, 0x45, 0x46, 0x45, 0x7d, 0x38, 0x00, // [213] 'Õ'
   0x38, 0x7c, 0x45, 0x44, 0x45, 0x7c, 0x38, 0x00, // [214] 'Ö'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [215] 'FREE'
   0x5c, 0x3e, 0x73, 0x49, 0x67, 0x3e, 0x1d, 0x00, // [216] 'Ø'
   0x7c, 0x7d, 0x41, 0x42, 0x7c, 0x7c, 0x00, 0x00, // [217] 'Ù'
   0x7c, 0x7c, 0x42, 0x41, 0x7d, 0x7c, 0x00, 0x00, // [218] 'Ú'
   0x7c, 0x7e, 0x41, 0x41, 0x7e, 0x7c, 0x00, 0x00, // [219] 'Û'
   0x7c, 0x7d, 0x40, 0x40, 0x7d, 0x7c, 0x00, 0x00, // [220] 'Ü'
   0x0c, 0x5c, 0x72, 0x71, 0x5d, 0x0c, 0x00, 0x00, // [221] 'Ý'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [222] 'FREE'
   0x00, 0xff, 0x7f, 0x01, 0x45, 0x7f, 0x3a, 0x00, // [223] 'ß'
   0x20, 0x75, 0x55, 0x56, 0x3c, 0x78, 0x40, 0x00, // [224] 'à'
   0x20, 0x74, 0x56, 0x55, 0x3d, 0x78, 0x40, 0x00, // [225] 'á'
   0x20, 0x76, 0x55, 0x55, 0x3d, 0x7a, 0x40, 0x00, // [226] 'â'
   0x22, 0x75, 0x55, 0x56, 0x3d, 0x79, 0x40, 0x00, // [227] 'ã'
   0x20, 0x75, 0x54, 0x54, 0x3d, 0x78, 0x40, 0x00, // [228] 'ä'
   0x20, 0x74, 0x56, 0x55, 0x3e, 0x78, 0x40, 0x00, // [229] 'å'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [230] 'FREE'
   0x1c, 0xbe, 0xa2, 0x62, 0x36, 0x14, 0x00, 0x00, // [231] 'ç'
   0x38, 0x7d, 0x55, 0x56, 0x5c, 0x18, 0x00, 0x00, // [232] 'è'
   0x38, 0x7c, 0x56, 0x55, 0x5d, 0x18, 0x00, 0x00, // [233] 'é'
   0x3a, 0x7d, 0x55, 0x55, 0x5d, 0x1a, 0x00, 0x00, // [234] 'ê'
   0x38, 0x7d, 0x54, 0x54, 0x5d, 0x18, 0x00, 0x00, // [235] 'ë'
   0x01, 0x45, 0x7e, 0x7c, 0x40, 0x00, 0x00, 0x00, // [236] 'ì'
   0x00, 0x44, 0x7e, 0x7d, 0x41, 0x00, 0x00, 0x00, // [237] 'í'
   0x02, 0x45, 0x7d, 0x7d, 0x42, 0x00, 0x00, 0x00, // [238] 'î'
   0x00, 0x45, 0x7c, 0x7d, 0x40, 0x00, 0x00, 0x00, // [239] 'ï'
   0x30, 0x78, 0x48, 0x4a, 0x3f, 0x7f, 0x42, 0x00, // [240] 'ð'
   0x7e, 0x79, 0x05, 0x06, 0x7d, 0x79, 0x00, 0x00, // [241] 'ñ'
   0x38, 0x7d, 0x45, 0x46, 0x7c, 0x38, 0x00, 0x00, // [242] 'ò'
   0x38, 0x7c, 0x46, 0x45, 0x7d, 0x38, 0x00, 0x00, // [243] 'ó'
   0x3a, 0x7d, 0x45, 0x45, 0x7d, 0x3a, 0x00, 0x00, // [244] 'ô'
   0x3a, 0x7d, 0x45, 0x46, 0x7d, 0x39, 0x00, 0x00, // [245] 'õ'
   0x38, 0x7d, 0x44, 0x44, 0x7d, 0x38, 0x00, 0x00, // [246] 'ö'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [247] 'FREE'
   0xb8, 0x7c, 0x64, 0x54, 0x4c, 0x7c, 0x3a, 0x00, // [248] 'ø'
   0x3c, 0x7d, 0x41, 0x42, 0x3c, 0x7c, 0x40, 0x00, // [249] 'ù'
   0x3c, 0x7c, 0x42, 0x41, 0x3d, 0x7c, 0x40, 0x00, // [250] 'ú'
   0x3c, 0x7e, 0x41, 0x41, 0x3e, 0x7c, 0x40, 0x00, // [251] 'û'
   0x3c, 0x7d, 0x40, 0x40, 0x3d, 0x7c, 0x40, 0x00, // [252] 'ü'
   0x9c, 0xbc, 0xa2, 0xa1, 0xfd, 0x7c, 0x00, 0x00, // [253] 'ý'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [254] 'FREE'
   0x9c, 0xbd, 0xa0, 0xa0, 0xfd, 0x7c, 0x00, 0x00  // [255] 'ÿ'
};


const uint8_t font_8x16[] = {
   0x00, 0x00, // Ignored.
   32        , // first char ASCII code.
   255       , // last char ASCII code.
   8         , // width of the character in pixels.
   16        , // height of the character in pixels.
   1         , // number of line of bytes per character.
   16        , // bytes per character (normally width * height / 8).
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 32] ' '
   0x00, 0x00, 0x00, 0xFC, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x19, 0x00, 0x00, 0x00, // [ 33] '!'
   0x00, 0x00, 0x3C, 0x00, 0x00, 0x3C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 34] '"'
   0x00, 0x60, 0xE0, 0x7C, 0x60, 0xF8, 0x7C, 0x60, 0x03, 0x13, 0x1F, 0x03, 0x1F, 0x07, 0x03, 0x00, // [ 35] '#'
   0x70, 0xF8, 0xC8, 0x8E, 0x88, 0x38, 0x30, 0x00, 0x0C, 0x1C, 0x31, 0x71, 0x33, 0x1F, 0x0E, 0x00, // [ 36] '$'
   0x38, 0x7C, 0x44, 0x7C, 0xC0, 0x38, 0x08, 0x00, 0x00, 0x00, 0x04, 0x03, 0x0E, 0x1F, 0x11, 0x1F, // [ 37] '%'
   0x30, 0xFC, 0xCC, 0xCC, 0x78, 0x00, 0x00, 0x00, 0x0F, 0x1B, 0x19, 0x1B, 0x1E, 0x1E, 0x17, 0x00, // [ 38] '&'
   0x00, 0x00, 0x00, 0x1E, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 39] '''
   0x00, 0x00, 0xE0, 0xF8, 0x0C, 0x02, 0x00, 0x00, 0x00, 0x00, 0x07, 0x1F, 0x30, 0x40, 0x00, 0x00, // [ 40] '('
   0x00, 0x00, 0x02, 0x0C, 0xF8, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x40, 0x30, 0x1F, 0x07, 0x00, 0x00, // [ 41] ')'
   0x60, 0x60, 0xC0, 0xF8, 0xC0, 0x60, 0x60, 0x00, 0x00, 0x02, 0x03, 0x00, 0x03, 0x02, 0x00, 0x00, // [ 42] '*'
   0x80, 0x80, 0xF0, 0xF0, 0x80, 0x80, 0x80, 0x00, 0x01, 0x01, 0x0F, 0x0F, 0x01, 0x01, 0x01, 0x00, // [ 43] '+'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x7C, 0x1C, 0x00, 0x00, 0x00, 0x00, // [ 44] ','
   0x00, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, // [ 45] '-'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1C, 0x1C, 0x00, 0x00, 0x00, 0x00, // [ 46] '.'
   0x00, 0x00, 0x00, 0x80, 0xF0, 0x7C, 0x0C, 0x00, 0x00, 0x00, 0x1C, 0x1F, 0x03, 0x00, 0x00, 0x00, // [ 47] '/'
   0xF0, 0xF8, 0x9C, 0x8C, 0x5C, 0xF8, 0xF0, 0x00, 0x07, 0x0F, 0x1D, 0x18, 0x1C, 0x0F, 0x07, 0x00, // [ 48] '0'
   0x00, 0x30, 0x18, 0x18, 0xFC, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x1F, 0x00, 0x00, // [ 49] '1'
   0x00, 0x38, 0x3C, 0x0C, 0x8C, 0xFC, 0x78, 0x00, 0x00, 0x18, 0x1E, 0x1F, 0x1B, 0x19, 0x18, 0x00, // [ 50] '2'
   0x00, 0x18, 0x1C, 0x8C, 0x8C, 0xFC, 0x78, 0x00, 0x00, 0x0C, 0x1C, 0x19, 0x19, 0x1F, 0x0F, 0x00, // [ 51] '3'
   0x00, 0xC0, 0x70, 0x1C, 0xFC, 0xFC, 0x00, 0x00, 0x07, 0x07, 0x06, 0x06, 0x1F, 0x1F, 0x06, 0x00, // [ 52] '4'
   0x00, 0x7C, 0x7C, 0x6C, 0x6C, 0xEC, 0xCC, 0x00, 0x00, 0x0C, 0x1C, 0x18, 0x18, 0x1F, 0x0F, 0x00, // [ 53] '5'
   0x00, 0xF0, 0xF8, 0xDC, 0xCC, 0xCC, 0x80, 0x00, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x1F, 0x0F, 0x00, // [ 54] '6'
   0x00, 0x0C, 0x0C, 0x0C, 0xCC, 0xFC, 0x3C, 0x00, 0x00, 0x00, 0x18, 0x1F, 0x07, 0x00, 0x00, 0x00, // [ 55] '7'
   0x00, 0x78, 0xFC, 0xCC, 0xCC, 0xFC, 0x78, 0x00, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x1F, 0x0F, 0x00, // [ 56] '8'
   0x00, 0xF8, 0xFC, 0x0C, 0x0C, 0xFC, 0xF8, 0x00, 0x00, 0x01, 0x19, 0x19, 0x19, 0x0F, 0x07, 0x00, // [ 57] '9'
   0x00, 0x00, 0x00, 0x70, 0x70, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1C, 0x1C, 0x08, 0x00, 0x00, // [ 58] ':'
   0x00, 0x00, 0x00, 0x70, 0x70, 0x20, 0x00, 0x00, 0x00, 0x00, 0x40, 0x7C, 0x1C, 0x00, 0x00, 0x00, // [ 59] ';'
   0x00, 0x80, 0xC0, 0x60, 0x30, 0x18, 0x08, 0x00, 0x00, 0x00, 0x01, 0x03, 0x06, 0x0C, 0x08, 0x00, // [ 60] '<'
   0x60, 0x60, 0x60, 0x60, 0x60, 0x60, 0x60, 0x00, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x00, // [ 61] '='
   0x00, 0x08, 0x18, 0x30, 0x60, 0xC0, 0x80, 0x00, 0x00, 0x08, 0x0C, 0x06, 0x03, 0x01, 0x00, 0x00, // [ 62] '>'
   0x18, 0x1C, 0x0C, 0x8C, 0xCC, 0xF8, 0x30, 0x00, 0x00, 0x00, 0x12, 0x1B, 0x01, 0x00, 0x00, 0x00, // [ 63] '?'
   0xC0, 0x70, 0x88, 0xE4, 0x24, 0xE4, 0x08, 0xF0, 0x03, 0x0E, 0x1B, 0x16, 0x14, 0x17, 0x04, 0x03, // [ 64] '@'
   0x00, 0x80, 0xF0, 0x3C, 0xF8, 0xC0, 0x00, 0x00, 0x1C, 0x1F, 0x0F, 0x06, 0x07, 0x1F, 0x1C, 0x00, // [ 65] 'A'
   0xFC, 0xFC, 0x8C, 0x8C, 0x8C, 0xF8, 0x70, 0x00, 0x1F, 0x1F, 0x19, 0x19, 0x19, 0x1F, 0x0F, 0x00, // [ 66] 'B'
   0xF8, 0xFC, 0x0C, 0x0C, 0x0C, 0x38, 0x30, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0C, 0x04, 0x00, // [ 67] 'C'
   0xFC, 0xFC, 0x0C, 0x0C, 0x18, 0xF8, 0xE0, 0x00, 0x1F, 0x1F, 0x18, 0x18, 0x1C, 0x0F, 0x03, 0x00, // [ 68] 'D'
   0xFC, 0xFC, 0x8C, 0x8C, 0x8C, 0x8C, 0x0C, 0x00, 0x1F, 0x1F, 0x19, 0x19, 0x19, 0x19, 0x18, 0x00, // [ 69] 'E'
   0xFC, 0xFC, 0x8C, 0x8C, 0x8C, 0x8C, 0x0C, 0x00, 0x1F, 0x1F, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [ 70] 'F'
   0xF8, 0xFC, 0x0C, 0x8C, 0x8C, 0x9C, 0x90, 0x00, 0x0F, 0x1F, 0x18, 0x19, 0x19, 0x1F, 0x0F, 0x00, // [ 71] 'G'
   0xFC, 0xFC, 0x80, 0x80, 0x80, 0xFC, 0xFC, 0x00, 0x1F, 0x1F, 0x01, 0x01, 0x01, 0x1F, 0x1F, 0x00, // [ 72] 'H'
   0x00, 0x0C, 0x0C, 0xFC, 0xFC, 0x0C, 0x0C, 0x00, 0x00, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [ 73] 'I'
   0x00, 0x00, 0x00, 0x00, 0x00, 0xFC, 0xFC, 0x00, 0x0C, 0x1C, 0x18, 0x18, 0x1C, 0x0F, 0x03, 0x00, // [ 74] 'J'
   0xFC, 0xFC, 0xC0, 0xE0, 0x38, 0x1C, 0x04, 0x00, 0x1F, 0x1F, 0x01, 0x03, 0x0F, 0x1E, 0x18, 0x00, // [ 75] 'K'
   0xFC, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x1F, 0x18, 0x18, 0x18, 0x18, 0x18, 0x00, // [ 76] 'L'
   0xFC, 0x3C, 0xF8, 0xE0, 0x3C, 0xFC, 0xFC, 0x00, 0x1F, 0x1E, 0x01, 0x01, 0x00, 0x1F, 0x1F, 0x00, // [ 77] 'M'
   0xFC, 0xFC, 0xF0, 0xC0, 0x00, 0xFC, 0xFC, 0x00, 0x1F, 0x1F, 0x01, 0x03, 0x06, 0x1F, 0x1F, 0x00, // [ 78] 'N'
   0xF8, 0xFC, 0x0C, 0x0C, 0x0C, 0xF8, 0xF0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x1F, 0x07, 0x00, // [ 79] 'O'
   0xFC, 0xFC, 0x8C, 0x8C, 0x8C, 0xF8, 0x70, 0x00, 0x1F, 0x1F, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [ 80] 'P'
   0xF8, 0xFC, 0x0C, 0x0C, 0x0C, 0xF8, 0xF0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x3F, 0x67, 0x00, // [ 81] 'Q'
   0xFC, 0xFC, 0x8C, 0x8C, 0x8C, 0xF8, 0x70, 0x00, 0x1F, 0x1F, 0x01, 0x03, 0x0F, 0x1E, 0x10, 0x00, // [ 82] 'R'
   0x78, 0xFC, 0xCC, 0x8C, 0x8C, 0x98, 0x10, 0x00, 0x0C, 0x1C, 0x18, 0x18, 0x19, 0x1F, 0x0F, 0x00, // [ 83] 'S'
   0x0C, 0x0C, 0xFC, 0xFC, 0x0C, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x1F, 0x00, 0x00, 0x00, 0x00, // [ 84] 'T'
   0xFC, 0xFC, 0x00, 0x00, 0x00, 0xFC, 0xFC, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x1F, 0x07, 0x00, // [ 85] 'U'
   0x3C, 0xFC, 0xC0, 0x00, 0xC0, 0xFC, 0x3C, 0x00, 0x00, 0x01, 0x07, 0x1C, 0x07, 0x01, 0x00, 0x00, // [ 86] 'V'
   0xFC, 0x7C, 0x80, 0x80, 0x00, 0xFC, 0xFC, 0x00, 0x1F, 0x1E, 0x07, 0x03, 0x1E, 0x1F, 0x0F, 0x00, // [ 87] 'W'
   0x0C, 0x3C, 0xF0, 0xE0, 0x38, 0x0C, 0x04, 0x00, 0x18, 0x1E, 0x07, 0x03, 0x0E, 0x1C, 0x18, 0x00, // [ 88] 'X'
   0x1C, 0x7C, 0xE0, 0xC0, 0xF0, 0x3C, 0x0C, 0x00, 0x00, 0x00, 0x1F, 0x1F, 0x01, 0x00, 0x00, 0x00, // [ 89] 'Y'
   0x0C, 0x0C, 0x8C, 0xEC, 0x7C, 0x1C, 0x0C, 0x00, 0x1C, 0x1E, 0x1B, 0x19, 0x18, 0x18, 0x18, 0x00, // [ 90] 'Z'
   0x00, 0x00, 0x00, 0xFE, 0xFE, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x7F, 0x60, 0x00, 0x00, // [ 91] '['
   0x00, 0x00, 0x1C, 0xFC, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x1F, 0x10, 0x00, // [ 92] '\'
   0x00, 0x00, 0x02, 0xFE, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x7F, 0x7F, 0x00, 0x00, 0x00, // [ 93] ']'
   0x00, 0x08, 0x04, 0x02, 0x04, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 94] '^'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x60, 0x60, 0x60, 0x60, 0x60, 0x00, // [ 95] '_'
   0x00, 0x00, 0x06, 0x0E, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 96] '`'
   0x40, 0x60, 0x60, 0x60, 0xE0, 0xC0, 0x00, 0x00, 0x0E, 0x1F, 0x19, 0x19, 0x1F, 0x1F, 0x10, 0x00, // [ 97] 'a'
   0xFC, 0xFC, 0x60, 0x60, 0x60, 0xC0, 0x00, 0x00, 0x1F, 0x1F, 0x18, 0x18, 0x18, 0x0F, 0x03, 0x00, // [ 98] 'b'
   0x00, 0xC0, 0xE0, 0x60, 0x60, 0x60, 0xC0, 0x00, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0C, 0x00, // [ 99] 'c'
   0x00, 0xC0, 0xE0, 0x60, 0x60, 0xFC, 0xFC, 0x00, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x0F, 0x1F, 0x00, // [100] 'd'
   0xC0, 0xC0, 0x60, 0x60, 0x60, 0xC0, 0x80, 0x00, 0x0F, 0x0F, 0x1B, 0x1B, 0x1B, 0x1B, 0x0B, 0x00, // [101] 'e'
   0x60, 0x60, 0xFC, 0xFC, 0x66, 0x66, 0x04, 0x00, 0x00, 0x00, 0x1F, 0x1F, 0x00, 0x00, 0x00, 0x00, // [102] 'f'
   0xC0, 0xE0, 0x60, 0x60, 0x40, 0xE0, 0xC0, 0x00, 0x4F, 0xDF, 0xD8, 0xD8, 0xE8, 0x7F, 0x1F, 0x00, // [103] 'g'
   0xFC, 0xFC, 0x40, 0x60, 0x60, 0xE0, 0x80, 0x00, 0x1F, 0x1F, 0x00, 0x00, 0x00, 0x1F, 0x1F, 0x00, // [104] 'h'
   0x60, 0x60, 0x64, 0xEC, 0xE4, 0x00, 0x00, 0x00, 0x18, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [105] 'i'
   0x00, 0x00, 0x60, 0x60, 0xEC, 0xEC, 0x00, 0x00, 0x40, 0xC0, 0xC0, 0xC0, 0xFF, 0x7F, 0x00, 0x00, // [106] 'j'
   0xFC, 0xFC, 0x80, 0xC0, 0xE0, 0x60, 0x20, 0x00, 0x1F, 0x1F, 0x03, 0x07, 0x0E, 0x18, 0x10, 0x00, // [107] 'k'
   0x0C, 0x0C, 0x0C, 0xFC, 0xFC, 0x00, 0x00, 0x00, 0x18, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [108] 'l'
   0xE0, 0x60, 0xC0, 0xE0, 0x60, 0xE0, 0xC0, 0x00, 0x1F, 0x1E, 0x01, 0x01, 0x00, 0x1F, 0x1F, 0x00, // [109] 'm'
   0xE0, 0xE0, 0x60, 0x60, 0x60, 0xE0, 0x80, 0x00, 0x1F, 0x1F, 0x00, 0x00, 0x00, 0x1F, 0x1F, 0x00, // [110] 'n'
   0xC0, 0xE0, 0x60, 0x60, 0x60, 0xC0, 0x80, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0F, 0x07, 0x00, // [111] 'o'
   0xE0, 0xE0, 0x60, 0x60, 0xE0, 0xC0, 0x00, 0x00, 0xFF, 0xFF, 0x18, 0x18, 0x1C, 0x0F, 0x03, 0x00, // [112] 'p'
   0x00, 0xC0, 0xE0, 0x60, 0x60, 0xC0, 0xE0, 0x00, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0xFF, 0xFF, 0x00, // [113] 'q'
   0x00, 0xE0, 0xE0, 0x40, 0x60, 0x60, 0x20, 0x00, 0x00, 0x1F, 0x1F, 0x00, 0x00, 0x00, 0x00, 0x00, // [114] 'r'
   0xC0, 0xE0, 0x60, 0x60, 0x60, 0xC0, 0x80, 0x00, 0x0C, 0x1D, 0x19, 0x1B, 0x1A, 0x0E, 0x04, 0x00, // [115] 's'
   0x60, 0xF8, 0x78, 0x60, 0x60, 0x60, 0x00, 0x00, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x18, 0x00, // [116] 't'
   0xE0, 0xE0, 0x00, 0x00, 0x00, 0xE0, 0xE0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x08, 0x1F, 0x1F, 0x00, // [117] 'u'
   0xE0, 0xE0, 0x00, 0x00, 0xC0, 0xE0, 0x20, 0x00, 0x00, 0x07, 0x1E, 0x1C, 0x0F, 0x01, 0x00, 0x00, // [118] 'v'
   0xE0, 0xE0, 0x00, 0x00, 0x00, 0xE0, 0xE0, 0x00, 0x1F, 0x19, 0x0E, 0x1E, 0x18, 0x1F, 0x0F, 0x00, // [119] 'w'
   0x20, 0xE0, 0xC0, 0x80, 0xC0, 0x60, 0x20, 0x00, 0x10, 0x1C, 0x0F, 0x07, 0x0F, 0x1C, 0x10, 0x00, // [120] 'x'
   0xE0, 0xE0, 0x00, 0x00, 0x80, 0xE0, 0x60, 0x00, 0xC0, 0xC3, 0xFF, 0x3C, 0x0F, 0x03, 0x00, 0x00, // [121] 'y'
   0x60, 0x60, 0x60, 0xE0, 0xE0, 0xE0, 0x20, 0x00, 0x18, 0x1C, 0x1F, 0x1B, 0x19, 0x18, 0x18, 0x00, // [122] 'z'
   0x00, 0x80, 0xC0, 0x7C, 0x3E, 0x02, 0x00, 0x00, 0x00, 0x00, 0x01, 0x3F, 0x7E, 0x40, 0x00, 0x00, // [123] '{'
   0x00, 0x00, 0x00, 0x7E, 0x7E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7E, 0x7E, 0x00, 0x00, 0x00, // [124] '|'
   0x00, 0x00, 0x02, 0x3E, 0x7C, 0xC0, 0x80, 0x00, 0x00, 0x00, 0x40, 0x7E, 0x3F, 0x01, 0x00, 0x00, // [125] '}'
   0x00, 0x04, 0x02, 0x02, 0x04, 0x08, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [126] '~'
   0xF0, 0xFA, 0x1E, 0x1C, 0x1E, 0x72, 0x60, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0C, 0x04, 0x00, // [127] '*Č'
   0x00, 0xC0, 0xE8, 0x78, 0x70, 0x78, 0xC8, 0x00, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0C, 0x00, // [128] '*č'
   0xF0, 0xF8, 0x1C, 0x1A, 0x1A, 0x70, 0x60, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0C, 0x04, 0x00, // [129] '*Ć'
   0x00, 0xC0, 0xE0, 0x70, 0x68, 0x68, 0xC0, 0x00, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0C, 0x00, // [130] '*ć'
   0x00, 0x80, 0xF0, 0x3C, 0xF8, 0xC0, 0x00, 0x00, 0x1C, 0x1F, 0x0F, 0x06, 0x67, 0x5F, 0x5C, 0x00, // [131] '*Ą'
   0x40, 0x60, 0x60, 0x60, 0xE0, 0xC0, 0x00, 0x00, 0x0E, 0x1F, 0x19, 0x19, 0x7F, 0x5F, 0x50, 0x00, // [132] '*ą'
   0xFC, 0xFC, 0x8C, 0x8C, 0x8C, 0x8C, 0x0C, 0x00, 0x1F, 0x1F, 0x19, 0x19, 0x79, 0x59, 0x58, 0x00, // [133] '*Ę'
   0xC0, 0xC0, 0x60, 0x60, 0x60, 0xC0, 0x80, 0x00, 0x0F, 0x0F, 0x1B, 0x1B, 0x7B, 0x5B, 0x4B, 0x00, // [134] '*ę'
   0x00, 0x00, 0xFC, 0xFC, 0xC0, 0x60, 0x30, 0x00, 0x06, 0x03, 0x1F, 0x1F, 0x18, 0x18, 0x18, 0x00, // [135] '*Ł'
   0x0C, 0x0C, 0x8C, 0xFC, 0xFC, 0xC0, 0x60, 0x00, 0x18, 0x1B, 0x19, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [136] '*ł'
   0xF0, 0xF0, 0xC4, 0x02, 0x02, 0xF0, 0xF0, 0x00, 0x1F, 0x1F, 0x01, 0x03, 0x06, 0x1F, 0x1F, 0x00, // [137] '*Ń'
   0x70, 0xFA, 0xDE, 0x9C, 0x9E, 0xB2, 0x20, 0x00, 0x0C, 0x1C, 0x18, 0x18, 0x19, 0x1F, 0x0F, 0x00, // [138] '*Š'
   0xE0, 0xE0, 0x68, 0x64, 0x64, 0xE0, 0x80, 0x00, 0x1F, 0x1F, 0x00, 0x00, 0x00, 0x1F, 0x1F, 0x00, // [139] '*ń'
   0x70, 0xF8, 0xDA, 0x99, 0x99, 0xB0, 0x20, 0x00, 0x0C, 0x1C, 0x18, 0x18, 0x19, 0x1F, 0x0F, 0x00, // [140] '*Ś'
   0xC0, 0xE0, 0x68, 0x64, 0x64, 0xC0, 0x80, 0x00, 0x0C, 0x1D, 0x19, 0x1B, 0x1A, 0x0E, 0x04, 0x00, // [141] '*ś'
   0x18, 0x1A, 0x9E, 0xDC, 0x7E, 0x3A, 0x18, 0x00, 0x1C, 0x1E, 0x1B, 0x19, 0x18, 0x18, 0x18, 0x00, // [142] '*Ž'
   0x18, 0x18, 0x98, 0xDA, 0x79, 0x39, 0x18, 0x00, 0x1C, 0x1E, 0x1B, 0x19, 0x18, 0x18, 0x18, 0x00, // [143] '*Ź'
   0x60, 0x60, 0x60, 0x68, 0xE4, 0xE4, 0x20, 0x00, 0x18, 0x1C, 0x1E, 0x1B, 0x19, 0x18, 0x18, 0x00, // [144] '*ź'
   0x18, 0x18, 0x99, 0xDB, 0x79, 0x38, 0x18, 0x00, 0x1C, 0x1E, 0x1B, 0x19, 0x18, 0x18, 0x18, 0x00, // [145] '*Ż'
   0x60, 0x60, 0x64, 0x6C, 0xE4, 0xE0, 0x20, 0x00, 0x18, 0x1C, 0x1E, 0x1B, 0x19, 0x18, 0x18, 0x00, // [146] '*ż'
   0x60, 0x60, 0x60, 0xE0, 0xE0, 0x00, 0x00, 0x00, 0x18, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [147] '*ı'
   0x00, 0x30, 0x30, 0xF2, 0xF6, 0x32, 0x30, 0x00, 0x00, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [148] '*İ'
   0x78, 0xFC, 0xCC, 0x8C, 0x8C, 0x98, 0x10, 0x00, 0x0C, 0x1C, 0x58, 0x78, 0x19, 0x1F, 0x0F, 0x00, // [149] '*Ş'
   0xC0, 0xE0, 0x60, 0x60, 0x60, 0xC0, 0x80, 0x00, 0x0C, 0x5D, 0x59, 0x3B, 0x1A, 0x0E, 0x04, 0x00, // [150] '*ş'
   0xF0, 0xFA, 0x1E, 0x9C, 0x9E, 0xBA, 0xA0, 0x00, 0x0F, 0x1F, 0x18, 0x19, 0x19, 0x1F, 0x0F, 0x00, // [151] '*Ğ'
   0xC0, 0xE4, 0x6C, 0x68, 0x4C, 0xE4, 0xC0, 0x00, 0x4F, 0xDF, 0xD8, 0xD8, 0xE8, 0x7F, 0x1F, 0x00, // [152] '*ğ'
   0xFA, 0xFE, 0x9C, 0x9E, 0x9A, 0xF0, 0x60, 0x00, 0x1F, 0x1F, 0x01, 0x03, 0x0F, 0x1E, 0x10, 0x00, // [153] '*Ř'
   0xC0, 0xE8, 0x78, 0x70, 0x78, 0xC8, 0x80, 0x00, 0x0C, 0x1D, 0x19, 0x1B, 0x1A, 0x0E, 0x04, 0x00, // [154] '*š'
   0x00, 0xE4, 0xEC, 0x48, 0x6C, 0x64, 0x20, 0x00, 0x00, 0x1F, 0x1F, 0x00, 0x00, 0x00, 0x00, 0x00, // [155] '*ř'
   0xF8, 0xFA, 0x9E, 0x9C, 0x9E, 0x9A, 0x18, 0x00, 0x1F, 0x1F, 0x19, 0x19, 0x19, 0x19, 0x18, 0x00, // [156] '*Ě'
   0xC0, 0xC4, 0x6C, 0x68, 0x6C, 0xC4, 0x80, 0x00, 0x0F, 0x0F, 0x1B, 0x1B, 0x1B, 0x1B, 0x0B, 0x00, // [157] '*ě'
   0x60, 0x68, 0x78, 0x70, 0xF8, 0xE8, 0x20, 0x00, 0x18, 0x1C, 0x1E, 0x1B, 0x19, 0x18, 0x18, 0x00, // [158] '*ž'
   0x70, 0xF0, 0x84, 0x00, 0xC4, 0xF0, 0x30, 0x00, 0x00, 0x01, 0x1F, 0x1F, 0x03, 0x00, 0x00, 0x00, // [159] '*Ÿ'
   0xFC, 0xFC, 0x07, 0x05, 0x07, 0xFC, 0xFC, 0x00, 0x03, 0x0F, 0x1C, 0x18, 0x1C, 0x0F, 0x03, 0x00, // [160] '*Ů'
   0xE0, 0xE0, 0x08, 0x14, 0x08, 0xE0, 0xE0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x08, 0x1F, 0x1F, 0x00, // [161] '*ů'
   0xE0, 0xF0, 0x34, 0x32, 0x34, 0xE2, 0xC0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x1F, 0x07, 0x00, // [162] '*Ő'
   0xC0, 0xE0, 0x68, 0x64, 0x68, 0xC4, 0x80, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0F, 0x07, 0x00, // [163] '*ő'
   0xF0, 0xF0, 0x04, 0x02, 0x04, 0xF2, 0xF0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x1F, 0x07, 0x00, // [164] '*Ű'
   0xE0, 0xE0, 0x08, 0x04, 0x08, 0xE4, 0xE0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x08, 0x1F, 0x1F, 0x00, // [165] '*ű'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [166] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [167] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [168] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [169] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [170] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [171] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [172] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [173] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [174] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [175] 'FREE'
   0x00, 0x06, 0x09, 0x09, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [176] '°'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [177] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [178] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [179] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [180] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [181] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [182] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [183] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [184] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [185] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [186] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [187] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [188] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [189] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [190] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [191] 'FREE'
   0x00, 0x00, 0xE2, 0x7A, 0xF4, 0x80, 0x00, 0x00, 0x1C, 0x1F, 0x0F, 0x06, 0x07, 0x1F, 0x1C, 0x00, // [192] 'À'
   0x00, 0x00, 0xE4, 0x7A, 0xF2, 0x80, 0x00, 0x00, 0x1C, 0x1F, 0x0F, 0x06, 0x07, 0x1F, 0x1C, 0x00, // [193] 'Á'
   0x00, 0x00, 0xE4, 0x72, 0xF4, 0x80, 0x00, 0x00, 0x1C, 0x1F, 0x0F, 0x06, 0x07, 0x1F, 0x1C, 0x00, // [194] 'Â'
   0x00, 0x04, 0xE2, 0x7A, 0xF4, 0x82, 0x00, 0x00, 0x1C, 0x1F, 0x0F, 0x06, 0x07, 0x1F, 0x1C, 0x00, // [195] 'Ã'
   0x00, 0x00, 0xE2, 0x78, 0xF2, 0x80, 0x00, 0x00, 0x1C, 0x1F, 0x0F, 0x06, 0x07, 0x1F, 0x1C, 0x00, // [196] 'Ä'
   0x00, 0x00, 0xEE, 0x7A, 0xEE, 0x80, 0x00, 0x00, 0x1C, 0x1F, 0x0F, 0x06, 0x07, 0x1F, 0x1C, 0x00, // [197] 'Å'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [198] 'FREE'
   0xF8, 0xFC, 0x0C, 0x0C, 0x0C, 0x38, 0x30, 0x00, 0x0F, 0x1F, 0x58, 0x78, 0x18, 0x0C, 0x04, 0x00, // [199] 'Ç'
   0xF0, 0xF0, 0xB2, 0xB2, 0xB4, 0xB0, 0x30, 0x00, 0x1F, 0x1F, 0x19, 0x19, 0x19, 0x19, 0x18, 0x00, // [200] 'È'
   0xF0, 0xF0, 0xB4, 0xB2, 0xB2, 0xB0, 0x30, 0x00, 0x1F, 0x1F, 0x19, 0x19, 0x19, 0x19, 0x18, 0x00, // [201] 'É'
   0xF0, 0xF0, 0xB4, 0xB2, 0xB4, 0xB0, 0x30, 0x00, 0x1F, 0x1F, 0x19, 0x19, 0x19, 0x19, 0x18, 0x00, // [202] 'Ê'
   0xF0, 0xF0, 0xB4, 0xB0, 0xB4, 0xB0, 0x30, 0x00, 0x1F, 0x1F, 0x19, 0x19, 0x19, 0x19, 0x18, 0x00, // [203] 'Ë'
   0x00, 0x30, 0x32, 0xF2, 0xF4, 0x30, 0x30, 0x00, 0x00, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [204] 'Ì'
   0x00, 0x30, 0x34, 0xF2, 0xF2, 0x30, 0x30, 0x00, 0x00, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [205] 'Í'
   0x00, 0x30, 0x34, 0xF2, 0xF4, 0x30, 0x30, 0x00, 0x00, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [206] 'Î'
   0x00, 0x30, 0x34, 0xF0, 0xF4, 0x30, 0x30, 0x00, 0x00, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [207] 'Ï'
   0xFC, 0xFC, 0x8C, 0x8C, 0x18, 0xF8, 0xE0, 0x00, 0x1F, 0x1F, 0x18, 0x18, 0x1C, 0x0F, 0x03, 0x00, // [208] 'Ð'
   0xF0, 0xF4, 0xC2, 0x02, 0x04, 0xF2, 0xF0, 0x00, 0x1F, 0x1F, 0x01, 0x03, 0x06, 0x1F, 0x1F, 0x00, // [209] 'Ñ'
   0xE0, 0xF0, 0x32, 0x32, 0x34, 0xE0, 0xC0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x1F, 0x07, 0x00, // [210] 'Ò'
   0xE0, 0xF0, 0x34, 0x32, 0x32, 0xE0, 0xC0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x1F, 0x07, 0x00, // [211] 'Ó'
   0xE0, 0xF0, 0x34, 0x32, 0x34, 0xE0, 0xC0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x1F, 0x07, 0x00, // [212] 'Ô'
   0xE0, 0xF4, 0x32, 0x32, 0x34, 0xE2, 0xC0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x1F, 0x07, 0x00, // [213] 'Õ'
   0xE0, 0xF0, 0x34, 0x30, 0x34, 0xE0, 0xC0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x1F, 0x07, 0x00, // [214] 'Ö'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [215] 'FREE'
   0xF8, 0xFC, 0x0C, 0xCC, 0x2C, 0xF8, 0xF8, 0x00, 0x0F, 0x1F, 0x1A, 0x19, 0x18, 0x1F, 0x07, 0x00, // [216] 'Ø'
   0xF0, 0xF0, 0x02, 0x02, 0x04, 0xF0, 0xF0, 0x00, 0x03, 0x0F, 0x1C, 0x18, 0x1C, 0x0F, 0x03, 0x00, // [217] 'Ù'
   0xF0, 0xF0, 0x04, 0x02, 0x02, 0xF0, 0xF0, 0x00, 0x03, 0x0F, 0x1C, 0x18, 0x1C, 0x0F, 0x03, 0x00, // [218] 'Ú'
   0xF0, 0xF0, 0x04, 0x02, 0x04, 0xF0, 0xF0, 0x00, 0x03, 0x0F, 0x1C, 0x18, 0x1C, 0x0F, 0x03, 0x00, // [219] 'Û'
   0xF0, 0xF0, 0x04, 0x00, 0x04, 0xF0, 0xF0, 0x00, 0x03, 0x0F, 0x1C, 0x18, 0x1C, 0x0F, 0x03, 0x00, // [220] 'Ü'
   0x70, 0xF0, 0x84, 0x02, 0xC2, 0xF0, 0x30, 0x00, 0x00, 0x01, 0x1F, 0x1F, 0x03, 0x00, 0x00, 0x00, // [221] 'Ý'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [222] 'FREE'
   0xFC, 0xFC, 0x0C, 0xCC, 0xCC, 0xF8, 0xB0, 0x00, 0x7F, 0x3F, 0x00, 0x38, 0x1C, 0x0F, 0x07, 0x00, // [223] 'ß'
   0x40, 0x64, 0x64, 0x68, 0xE0, 0xC0, 0x00, 0x00, 0x0E, 0x1F, 0x19, 0x19, 0x1F, 0x1F, 0x10, 0x00, // [224] 'à'
   0x40, 0x60, 0x60, 0x68, 0xE4, 0xC4, 0x00, 0x00, 0x0E, 0x1F, 0x19, 0x19, 0x1F, 0x1F, 0x10, 0x00, // [225] 'á'
   0x40, 0x60, 0x68, 0x64, 0xE8, 0xC0, 0x00, 0x00, 0x0E, 0x1F, 0x19, 0x19, 0x1F, 0x1F, 0x10, 0x00, // [226] 'â'
   0x40, 0x68, 0x64, 0x64, 0xE8, 0xC4, 0x00, 0x00, 0x0E, 0x1F, 0x19, 0x19, 0x1F, 0x1F, 0x10, 0x00, // [227] 'ã'
   0x40, 0x60, 0x68, 0x60, 0xE8, 0xC0, 0x00, 0x00, 0x0E, 0x1F, 0x19, 0x19, 0x1F, 0x1F, 0x10, 0x00, // [228] 'ä'
   0x40, 0x60, 0x68, 0x74, 0xE8, 0xC0, 0x00, 0x00, 0x0E, 0x1F, 0x19, 0x19, 0x1F, 0x1F, 0x10, 0x00, // [229] 'å'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [230] 'FREE'
   0x00, 0xC0, 0xE0, 0x60, 0x60, 0x60, 0xC0, 0x00, 0x00, 0x0F, 0x5F, 0x58, 0x38, 0x18, 0x0C, 0x00, // [231] 'ç'
   0xC0, 0xC4, 0x64, 0x68, 0x60, 0xC0, 0x80, 0x00, 0x0F, 0x0F, 0x1B, 0x1B, 0x1B, 0x1B, 0x0B, 0x00, // [232] 'è'
   0xC0, 0xC0, 0x60, 0x68, 0x64, 0xC4, 0x80, 0x00, 0x0F, 0x0F, 0x1B, 0x1B, 0x1B, 0x1B, 0x0B, 0x00, // [233] 'é'
   0xC0, 0xC0, 0x68, 0x64, 0x68, 0xC0, 0x80, 0x00, 0x0F, 0x0F, 0x1B, 0x1B, 0x1B, 0x1B, 0x0B, 0x00, // [234] 'ê'
   0xC0, 0xC0, 0x68, 0x60, 0x68, 0xC0, 0x80, 0x00, 0x0F, 0x0F, 0x1B, 0x1B, 0x1B, 0x1B, 0x0B, 0x00, // [235] 'ë'
   0x60, 0x64, 0x64, 0xE8, 0xE0, 0x00, 0x00, 0x00, 0x18, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [236] 'ì'
   0x60, 0x68, 0x64, 0xE4, 0xE0, 0x00, 0x00, 0x00, 0x18, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [237] 'í'
   0x60, 0x60, 0x68, 0xE4, 0xE8, 0x00, 0x00, 0x00, 0x18, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [238] 'î'
   0x60, 0x68, 0x60, 0xE8, 0xE0, 0x00, 0x00, 0x00, 0x18, 0x18, 0x18, 0x1F, 0x1F, 0x18, 0x18, 0x00, // [239] 'ï'
   0x00, 0xC0, 0xE0, 0x60, 0x68, 0xFC, 0xFC, 0x08, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x0F, 0x1F, 0x00, // [240] 'ð'
   0xE0, 0xE8, 0x64, 0x64, 0x68, 0xE4, 0x80, 0x00, 0x1F, 0x1F, 0x00, 0x00, 0x00, 0x1F, 0x1F, 0x00, // [241] 'ñ'
   0xC0, 0xE4, 0x64, 0x68, 0x60, 0xC0, 0x80, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0F, 0x07, 0x00, // [242] 'ò'
   0xC0, 0xE8, 0x64, 0x64, 0x60, 0xC0, 0x80, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0F, 0x07, 0x00, // [243] 'ó'
   0xC0, 0xE0, 0x68, 0x64, 0x68, 0xC0, 0x80, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0F, 0x07, 0x00, // [244] 'ô'
   0xC0, 0xE8, 0x64, 0x64, 0x68, 0xC4, 0x80, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0F, 0x07, 0x00, // [245] 'õ'
   0xC0, 0xE0, 0x68, 0x60, 0x68, 0xC0, 0x80, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x18, 0x0F, 0x07, 0x00, // [246] 'ö'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [247] 'FREE'
   0xC0, 0xE0, 0x60, 0x60, 0xE0, 0xC0, 0xA0, 0x00, 0x2F, 0x1F, 0x1C, 0x1B, 0x18, 0x0F, 0x07, 0x00, // [248] 'ø'
   0xE0, 0xE4, 0x04, 0x08, 0x00, 0xE0, 0xE0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x08, 0x1F, 0x1F, 0x00, // [249] 'ù'
   0xE0, 0xE0, 0x00, 0x08, 0x04, 0xE4, 0xE0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x08, 0x1F, 0x1F, 0x00, // [250] 'ú'
   0xE0, 0xE0, 0x08, 0x04, 0x08, 0xE0, 0xE0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x08, 0x1F, 0x1F, 0x00, // [251] 'û'
   0xE0, 0xE0, 0x08, 0x00, 0x08, 0xE0, 0xE0, 0x00, 0x0F, 0x1F, 0x18, 0x18, 0x08, 0x1F, 0x1F, 0x00, // [252] 'ü'
   0xE0, 0xE0, 0x00, 0x10, 0x88, 0xE8, 0x60, 0x00, 0xC0, 0xC3, 0xFF, 0x3C, 0x0F, 0x03, 0x00, 0x00, // [253] 'ý'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [254] 'FREE'
   0xE0, 0xE0, 0x08, 0x00, 0x88, 0xE0, 0x60, 0x00, 0xC0, 0xC3, 0xFF, 0x3C, 0x0F, 0x03, 0x00, 0x00  // [255] 'ÿ'
};


const uint8_t font_16x32[] = {
   0x00, 0x00, // Ignored.
   32        , // first char ASCII code.
   255       , // last char ASCII code.
   16        , // width of the character in pixels.
   32        , // height of the character in pixels.
   4         , // number of line of bytes per character.
   64        , // bytes per character (normally width * height / 8).
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 32] ' '
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe3, 0xe3, 0xe3, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 33] '!'
   0x00, 0x00, 0x00, 0x00, 0xe0, 0xe0, 0x00, 0x00, 0x00, 0xe0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 34] '"'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x30, 0x30, 0xf0, 0xff, 0x7f, 0x33, 0x30, 0xf0, 0xff, 0x3f, 0x31, 0x30, 0x30, 0x00,
   0x00, 0x0c, 0x0c, 0xfc, 0xff, 0x1f, 0x0c, 0x0c, 0xfc, 0xff, 0x0f, 0x0c, 0x0c, 0x0c, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 35] '#'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0xf0, 0xf0, 0xf0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x18, 0x7e, 0xff, 0xff, 0xc7, 0xc3, 0x83, 0x83, 0x07, 0x1f, 0x1f, 0x1e, 0x00, 0x00,
   0x00, 0x00, 0x70, 0xf0, 0xf0, 0xc1, 0x81, 0x83, 0x83, 0x83, 0xc7, 0xff, 0xfe, 0xfc, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x03, 0x0f, 0x0f, 0x03, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [ 36] '$'
   0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x06, 0x1f, 0x3f, 0x30, 0x30, 0x3f, 0x1f, 0xcf, 0xf0, 0x7c, 0x1f, 0x06, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x3c, 0x0f, 0x03, 0xfc, 0xfe, 0xce, 0x86, 0x86, 0xfe, 0xfc, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [ 37] '%'
   0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xe1, 0xe1, 0x71, 0x3f, 0x1f, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x3c, 0xfe, 0xff, 0xff, 0xc3, 0xc3, 0xc7, 0xdf, 0xfe, 0xfc, 0xf8, 0xff, 0xdf, 0x8f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, // [ 38] '&'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xe0, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 39] '''
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe0, 0xf0, 0x78, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xe0, 0xfe, 0xff, 0x1f, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x03, 0x3f, 0xff, 0xfc, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x07, 0x0e, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 40] '('
   0x00, 0x00, 0x00, 0x00, 0x1c, 0x38, 0xf0, 0xe0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0xff, 0xfe, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xfc, 0xff, 0x3f, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x1c, 0x0f, 0x07, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 41] ')'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x70, 0x60, 0xe0, 0xc0, 0xc0, 0xfe, 0xfe, 0x9e, 0xc0, 0xc0, 0xe0, 0x60, 0x50, 0x00,
   0x00, 0x00, 0x00, 0x10, 0x18, 0x3c, 0x1e, 0x07, 0x03, 0x07, 0x1e, 0x3c, 0x18, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 42] '*'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xfe, 0xfe, 0xfe, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x3f, 0x3f, 0x3f, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 43] '+'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x38, 0x3f, 0x1f, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 44] ','
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 45] '-'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xf0, 0xf0, 0xf0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 46] '.'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0, 0xe0, 0x20, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xf8, 0xff, 0x7f, 0x0f, 0x01, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x80, 0xf0, 0xfe, 0xff, 0x1f, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 47] '/'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xfe, 0xff, 0xff, 0x07, 0x81, 0x81, 0xc1, 0xe3, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0x7f, 0xff, 0xf7, 0xc3, 0xc1, 0xc0, 0xe0, 0xff, 0xff, 0x7f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 48] '0'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x1e, 0x0e, 0x0e, 0x07, 0x07, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 49] '1'
   0x00, 0x00, 0x00, 0x80, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x0e, 0x0f, 0x0f, 0x07, 0x01, 0x01, 0x01, 0xc1, 0xe3, 0xff, 0xff, 0x3e, 0x00, 0x00, 0x00,
   0x00, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xde, 0xcf, 0xc7, 0xc3, 0xc1, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [ 50] '2'
   0x00, 0x00, 0x00, 0x80, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x04, 0x07, 0x07, 0x07, 0xc3, 0xc1, 0xc1, 0xc1, 0xe3, 0xff, 0x7f, 0x3f, 0x0c, 0x00, 0x00,
   0x00, 0x30, 0x70, 0xf0, 0xf0, 0xe1, 0xc1, 0xc1, 0xc1, 0xc1, 0xff, 0xff, 0x7f, 0x3c, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 51] '3'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xc0, 0xe0, 0xf8, 0x7e, 0x1f, 0x07, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x1c, 0x1f, 0x1f, 0x1f, 0x1c, 0x1c, 0x1c, 0x1c, 0xff, 0xff, 0xff, 0x1c, 0x1c, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, // [ 52] '4'
   0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xfc, 0xff, 0xff, 0xff, 0x71, 0x71, 0x71, 0xf1, 0xe1, 0xe1, 0xc1, 0x00, 0x00, 0x00,
   0x00, 0x08, 0x70, 0xf0, 0xf0, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xff, 0xff, 0x7f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 53] '5'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xc0, 0xf8, 0xfe, 0xff, 0xef, 0x63, 0x73, 0x71, 0xf1, 0xe1, 0xe0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x0f, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xff, 0xff, 0x7f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 54] '6'
   0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x81, 0xe1, 0xfd, 0xff, 0x3f, 0x07, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xe0, 0xf8, 0xfe, 0x7f, 0x1f, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 55] '7'
   0x00, 0x00, 0x00, 0x80, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0x7f, 0xff, 0xe3, 0xc1, 0xc1, 0xc1, 0xe3, 0xff, 0x7f, 0x3f, 0x00, 0x00, 0x00,
   0x00, 0x18, 0x7e, 0xff, 0xff, 0xe3, 0xc1, 0xc1, 0xc1, 0xe3, 0xff, 0xff, 0x7e, 0x3c, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 56] '8'
   0x00, 0x00, 0x00, 0x80, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x7c, 0xff, 0xff, 0xff, 0x83, 0x01, 0x01, 0x01, 0x83, 0xff, 0xff, 0xfe, 0xf0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x03, 0xc3, 0xc7, 0xc7, 0xc7, 0xe3, 0xfb, 0x7f, 0x3f, 0x0f, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 57] '9'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x7c, 0x7c, 0x7c, 0x38, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xf0, 0xf0, 0xf0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 58] ':'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x7c, 0x7c, 0x7c, 0x38, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x38, 0x3f, 0x1f, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 59] ';'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xe0, 0xe0, 0x70, 0x70, 0x70, 0x38, 0x38, 0x3c, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x03, 0x03, 0x07, 0x07, 0x07, 0x0e, 0x0e, 0x1e, 0x1c, 0x1c, 0x3c, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 60] '<'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x70, 0x70, 0x70, 0x70, 0x70, 0x70, 0x70, 0x70, 0x70, 0x70, 0x70, 0x70, 0x00, 0x00,
   0x00, 0x00, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 61] '='
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x3c, 0x38, 0x38, 0x70, 0x70, 0x70, 0xe0, 0xe0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x3c, 0x1c, 0x1c, 0x1e, 0x0e, 0x0e, 0x07, 0x07, 0x07, 0x03, 0x03, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 62] '>'
   0x00, 0x00, 0x00, 0x80, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x06, 0x07, 0x07, 0x07, 0x01, 0x81, 0xc1, 0xe1, 0xff, 0x7f, 0x3f, 0x1e, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe6, 0xe7, 0xe7, 0xc7, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 63] '?'
   0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0x0f, 0x83, 0xe1, 0xf0, 0x18, 0x18, 0xf8, 0xf8, 0x01, 0x07, 0xfe, 0xf8, 0x00,
   0x00, 0x3f, 0x7f, 0xe0, 0xc7, 0x9f, 0x9c, 0x98, 0x88, 0x8f, 0xdf, 0x18, 0x1c, 0x0f, 0x01, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 64] '@'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x80, 0xf0, 0xfe, 0xff, 0x1f, 0x1f, 0xff, 0xfe, 0xf0, 0x80, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0xfc, 0xff, 0xff, 0x1f, 0x1d, 0x1c, 0x1c, 0x1d, 0x1f, 0xff, 0xff, 0xfc, 0xe0, 0x00,
   0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, // [ 65] 'A'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xe3, 0xff, 0x7f, 0x3f, 0x08, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc3, 0xff, 0xff, 0x7f, 0x3c, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, // [ 66] 'B'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x01, 0x03, 0x0f, 0x0f, 0x0e, 0x10, 0x00,
   0x00, 0x0f, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf8, 0x78, 0x38, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 67] 'C'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x01, 0x03, 0x0f, 0xff, 0xfe, 0xfc, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf8, 0x7f, 0x3f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 68] 'D'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0x01, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [ 69] 'E'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0x01, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 70] 'F'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x81, 0x81, 0x83, 0x8f, 0x8f, 0x8e, 0x00, 0x00,
   0x00, 0x0f, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc3, 0xc3, 0xc3, 0xff, 0xff, 0x7f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 71] 'G'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [ 72] 'H'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [ 73] 'I'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x08, 0x78, 0xf8, 0xf8, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xff, 0xff, 0x7f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 74] 'J'
   0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0x40, 0x00,
   0x00, 0xff, 0xff, 0xff, 0xc0, 0xe0, 0xf0, 0xfc, 0xfe, 0x1f, 0x0f, 0x03, 0x01, 0x00, 0x00, 0x00,
   0x00, 0xff, 0xff, 0xff, 0x07, 0x03, 0x03, 0x0f, 0x1f, 0x7f, 0xfc, 0xf8, 0xe0, 0x80, 0x00, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, // [ 75] 'K'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [ 76] 'L'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x3f, 0xff, 0xf0, 0xf8, 0xff, 0x1f, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x01, 0x07, 0x07, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [ 77] 'M'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x3f, 0xfe, 0xf0, 0xc0, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x03, 0x1f, 0x7f, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [ 78] 'N'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x03, 0x07, 0xff, 0xff, 0xfe, 0xe0, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf0, 0xff, 0x7f, 0x3f, 0x03, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 79] 'O'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x81, 0x81, 0x81, 0x81, 0x81, 0xc3, 0xff, 0xff, 0xff, 0x3c, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x01, 0x01, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 80] 'P'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x01, 0x07, 0xff, 0xff, 0xfe, 0xf0, 0x00,
   0x00, 0x0f, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xff, 0xff, 0x3f, 0x07, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x03, 0x07, 0x07, 0x0f, 0x06, 0x00, // [ 81] 'Q'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x81, 0x81, 0x81, 0x81, 0x81, 0xc3, 0xff, 0xff, 0x7f, 0x18, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x03, 0x03, 0x03, 0x07, 0x1f, 0x7f, 0xfd, 0xf0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, // [ 82] 'R'
   0x00, 0x00, 0x00, 0x80, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0x7f, 0xff, 0xf3, 0xe1, 0xc1, 0xc1, 0xc1, 0x83, 0x8f, 0x0f, 0x0f, 0x0c, 0x00,
   0x00, 0x18, 0x78, 0xf8, 0xf8, 0xe0, 0xc1, 0xc1, 0xc1, 0xc3, 0xe3, 0xff, 0xff, 0x7e, 0x3c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 83] 'S'
   0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00,
   0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 84] 'T'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xff, 0xff, 0x7f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 85] 'U'
   0x40, 0xc0, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00,
   0x00, 0x03, 0x1f, 0xff, 0xff, 0xf8, 0xc0, 0x00, 0x00, 0xe0, 0xfe, 0xff, 0x7f, 0x0f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x0f, 0x7f, 0xff, 0xfc, 0xfe, 0xff, 0x3f, 0x07, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 86] 'V'
   0x40, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00,
   0x00, 0x7f, 0xff, 0xff, 0xc0, 0xe0, 0xff, 0xff, 0xff, 0xfe, 0x00, 0xfc, 0xff, 0xff, 0x0f, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x03, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [ 87] 'W'
   0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0x00,
   0x00, 0x00, 0x03, 0x07, 0x1f, 0x7f, 0xfc, 0xf0, 0xf8, 0xfe, 0x7f, 0x1f, 0x07, 0x03, 0x00, 0x00,
   0x00, 0x80, 0xe0, 0xf8, 0xfc, 0x7f, 0x1f, 0x07, 0x07, 0x1f, 0x7f, 0xfc, 0xf0, 0xe0, 0x80, 0x00,
   0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, // [ 88] 'X'
   0x40, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x40,
   0x00, 0x00, 0x03, 0x1f, 0x7f, 0xfe, 0xf8, 0xe0, 0xe0, 0xf8, 0xfe, 0x7f, 0x1f, 0x03, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0xff, 0xff, 0xff, 0xff, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 89] 'Y'
   0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0xc1, 0xe1, 0xf9, 0xfd, 0x3f, 0x1f, 0x07, 0x03, 0x00, 0x00,
   0x00, 0xc0, 0xe0, 0xf8, 0xfc, 0xff, 0xdf, 0xc7, 0xc1, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, // [ 90] 'Z'
   0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0xf8, 0xf8, 0xf8, 0x18, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f, 0x0e, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 91] '['
   0x00, 0x00, 0x00, 0x20, 0xe0, 0xe0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x0f, 0x7f, 0xff, 0xf8, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x1f, 0xff, 0xfe, 0xf0, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [ 92] '\'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0xf8, 0xf8, 0xf8, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 0x0e, 0x0f, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 93] ']'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe0, 0xf0, 0xf0, 0xf0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x30, 0x3c, 0x3f, 0x07, 0x01, 0x01, 0x07, 0x3f, 0x3e, 0x30, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 94] '^'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x00, 0x00, // [ 95] '_'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x70, 0xf0, 0xe0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 96] '`'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x60, 0x70, 0x70, 0x38, 0x38, 0x38, 0x38, 0x78, 0xf0, 0xf0, 0xe0, 0x00, 0x00, 0x00,
   0x00, 0x10, 0xfc, 0xfe, 0xfe, 0xc7, 0xc3, 0xc3, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [ 97] 'a'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x70, 0x30, 0x38, 0x38, 0x38, 0xf8, 0xf0, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xff, 0x7f, 0x3f, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 98] 'b'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0xf0, 0x78, 0x38, 0x38, 0x38, 0x38, 0xf8, 0xf0, 0xe0, 0x80, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xf0, 0x70, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [ 99] 'c'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xe0, 0xf0, 0xf0, 0x78, 0x38, 0x38, 0x38, 0x30, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x0f, 0x7f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [100] 'd'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0xf0, 0x78, 0x38, 0x38, 0x38, 0x38, 0x78, 0xf0, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0x7f, 0xff, 0xf7, 0xc7, 0xc7, 0xc7, 0xc7, 0xc7, 0xe7, 0xe7, 0x47, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [101] 'e'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0x60, 0x00,
   0x00, 0x00, 0x38, 0x38, 0x38, 0xff, 0xff, 0xff, 0xff, 0x39, 0x38, 0x38, 0x38, 0x38, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [102] 'f'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xe0, 0xf0, 0xf0, 0x78, 0x38, 0x38, 0x38, 0x30, 0xf0, 0xf8, 0xf8, 0xf8, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x08, 0x1c, 0x1c, 0x39, 0x39, 0x39, 0x39, 0x3c, 0x1f, 0x1f, 0x0f, 0x03, 0x00, 0x00, // [103] 'g'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x60, 0x30, 0x38, 0x38, 0x38, 0xf8, 0xf8, 0xf0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [104] 'h'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x38, 0x38, 0x38, 0x39, 0xfb, 0xfb, 0xfb, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [105] 'i'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x38, 0x38, 0x38, 0xf9, 0xfb, 0xfb, 0xfb, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x38, 0x38, 0x38, 0x38, 0x3f, 0x1f, 0x1f, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, // [106] 'j'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0x38, 0x18, 0x08, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x1f, 0x0f, 0x0f, 0x3f, 0x7f, 0xf8, 0xf0, 0xe0, 0x80, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, // [107] 'k'
   0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [108] 'l'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xf8, 0xf8, 0x30, 0x38, 0xf8, 0xf8, 0xf0, 0x30, 0x38, 0xf8, 0xf8, 0xf0, 0x80, 0x00,
   0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, // [109] 'm'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xf8, 0xf8, 0xf8, 0x60, 0x30, 0x38, 0x38, 0x38, 0xf8, 0xf8, 0xf0, 0xe0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [110] 'n'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xf0, 0xf0, 0x78, 0x38, 0x38, 0x38, 0x38, 0xf0, 0xf0, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xff, 0x7f, 0x3f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [111] 'o'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xf8, 0xf8, 0xf8, 0x70, 0x30, 0x38, 0x38, 0x38, 0xf8, 0xf0, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf9, 0xff, 0x7f, 0x3f, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0x3f, 0x3f, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [112] 'p'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xe0, 0xf0, 0xf0, 0x78, 0x38, 0x38, 0x38, 0x30, 0xf0, 0xf8, 0xf8, 0xf8, 0x00, 0x00,
   0x00, 0x07, 0x7f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x3f, 0x3f, 0x3f, 0x3f, 0x00, 0x00, // [113] 'q'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xf8, 0xf8, 0xf8, 0xe0, 0xf0, 0x70, 0x78, 0x78, 0x78, 0x78, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [114] 'r'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xe0, 0xf0, 0xf0, 0x78, 0x38, 0x38, 0x38, 0x38, 0xf8, 0xf0, 0xf0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x71, 0xf3, 0xf3, 0xe7, 0xc6, 0xc6, 0xc6, 0xce, 0xee, 0xfc, 0xfc, 0x78, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [115] 's'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x38, 0x38, 0x38, 0xff, 0xff, 0xff, 0x38, 0x38, 0x38, 0x38, 0x38, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x7f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [116] 't'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xf8, 0xf8, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0xf8, 0xf8, 0xf8, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xf0, 0xc0, 0xc0, 0xc0, 0xc0, 0x7f, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [117] 'u'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x18, 0x78, 0xf8, 0xf8, 0xc0, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xf8, 0xf8, 0x38, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x03, 0x1f, 0x7f, 0xfe, 0xf0, 0xf8, 0xff, 0x3f, 0x0f, 0x01, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [118] 'v'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x08, 0xf8, 0xf8, 0xf8, 0x00, 0x00, 0xf8, 0xf8, 0xf8, 0x80, 0x00, 0xe0, 0xf8, 0xf8, 0x38, 0x00,
   0x00, 0x03, 0x7f, 0xff, 0xf8, 0xff, 0x3f, 0x01, 0x0f, 0xff, 0xf0, 0xff, 0xff, 0x0f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [119] 'w'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x08, 0x18, 0x38, 0xf8, 0xf0, 0xe0, 0xc0, 0x80, 0xe0, 0xf0, 0xf8, 0x78, 0x18, 0x08, 0x00,
   0x00, 0x00, 0x80, 0xe0, 0xf0, 0xfd, 0x7f, 0x1f, 0x1f, 0x3f, 0xff, 0xf0, 0xe0, 0x80, 0x00, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, // [120] 'x'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x18, 0x78, 0xf8, 0xf8, 0xe0, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xf8, 0xf8, 0xf8, 0x38, 0x08,
   0x00, 0x00, 0x00, 0x01, 0x0f, 0x3f, 0xff, 0xf8, 0xf8, 0xfe, 0x3f, 0x0f, 0x03, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x30, 0x38, 0x38, 0x3c, 0x1f, 0x0f, 0x07, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [121] 'y'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x38, 0x38, 0x38, 0x38, 0x38, 0x38, 0xb8, 0xf8, 0xf8, 0xf8, 0x78, 0x38, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xf0, 0xf8, 0xfc, 0xfe, 0xdf, 0xcf, 0xc3, 0xc1, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [122] 'z'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xe0, 0xf0, 0x78, 0x18, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xe0, 0x7f, 0x7f, 0x3f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x03, 0xff, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x07, 0x0f, 0x0c, 0x00, 0x00, 0x00, 0x00, // [123] '{'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3f, 0x3f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfc, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [124] '|'
   0x00, 0x00, 0x00, 0x00, 0x18, 0x78, 0xf0, 0xe0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3f, 0x7f, 0x7f, 0xe0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x0c, 0x0f, 0x07, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [125] '}'
   0x80, 0x80, 0xe0, 0xe0, 0x60, 0x60, 0xe0, 0xe0, 0x80, 0x80, 0xe0, 0xe0, 0x60, 0x60, 0x00, 0x00,
   0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [126] '~'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xc4, 0xcc, 0xd8, 0xd0, 0xd8, 0xcc, 0x84, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x01, 0x03, 0x0f, 0x0f, 0x0e, 0x10, 0x00,
   0x00, 0x0f, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf8, 0x78, 0x38, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [127] '*Č'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0xf0, 0x78, 0x39, 0x3b, 0x3a, 0x3b, 0xf9, 0xf0, 0xe0, 0x80, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xf0, 0x70, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [128] '*č'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xd0, 0xd8, 0xcc, 0xc4, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x01, 0x03, 0x0f, 0x0f, 0x0e, 0x10, 0x00,
   0x00, 0x0f, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf8, 0x78, 0x38, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [129] '*Ć'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0xf0, 0x78, 0x38, 0x3a, 0x3b, 0x39, 0xf8, 0xf0, 0xe0, 0x80, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xf0, 0x70, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [130] '*ć'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x80, 0xf0, 0xfe, 0xff, 0x1f, 0x1f, 0xff, 0xfe, 0xf0, 0x80, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0xfc, 0xff, 0xff, 0x1f, 0x1d, 0x1c, 0x1c, 0x1d, 0x1f, 0xff, 0xff, 0xfc, 0xe0, 0x00,
   0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x1f, 0x33, 0x21, 0x01, // [131] '*Ą'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x60, 0x70, 0x70, 0x38, 0x38, 0x38, 0x38, 0x78, 0xf0, 0xf0, 0xe0, 0x00, 0x00, 0x00,
   0x00, 0x10, 0xfc, 0xfe, 0xfe, 0xc7, 0xc3, 0xc3, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x0d, 0x1f, 0x33, 0x21, 0x00, 0x00, // [132] '*ą'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0x01, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x0d, 0x1f, 0x33, 0x21, 0x00, 0x00, // [133] '*Ę'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0xf0, 0x78, 0x38, 0x38, 0x38, 0x38, 0x78, 0xf0, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0x7f, 0xff, 0xf7, 0xc7, 0xc7, 0xc7, 0xc7, 0xc7, 0xe7, 0xe7, 0x47, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x0d, 0x1f, 0x33, 0x20, 0x00, 0x00, 0x00, // [134] '*ę'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x80, 0xc0, 0xc0, 0x60, 0x60, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x0c, 0x06, 0xff, 0xff, 0xff, 0xc1, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [135] '*Ł'
   0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x81, 0xc1, 0xc1, 0xff, 0xff, 0xff, 0x18, 0x1c, 0x0c, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xc0, 0xc1, 0xc1, 0xc0, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [136] '*ł'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x80, 0x10, 0x18, 0x0c, 0x0c, 0xcc, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x3f, 0xfe, 0xf0, 0xc0, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x03, 0x1f, 0x7f, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [137] '*Ń'
   0x00, 0x00, 0x00, 0x80, 0x80, 0xc4, 0xcc, 0xd8, 0xd0, 0xd8, 0xcc, 0x84, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0x7f, 0xff, 0xf3, 0xe1, 0xc1, 0xc1, 0xc1, 0x83, 0x8f, 0x0f, 0x0f, 0x0c, 0x00,
   0x00, 0x18, 0x78, 0xf8, 0xf8, 0xe0, 0xc1, 0xc1, 0xc1, 0xc3, 0xe3, 0xff, 0xff, 0x7e, 0x3c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [138] '*Š'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xf8, 0xf8, 0xf8, 0x60, 0x32, 0x3b, 0x39, 0x39, 0xf9, 0xf8, 0xf0, 0xe0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [139] '*ń'
   0x00, 0x00, 0x00, 0x80, 0x80, 0xc0, 0xd0, 0xd8, 0xcc, 0xcc, 0xcc, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0x7f, 0xff, 0xf3, 0xe1, 0xc1, 0xc1, 0xc1, 0x83, 0x8f, 0x0f, 0x0f, 0x0c, 0x00,
   0x00, 0x18, 0x78, 0xf8, 0xf8, 0xe0, 0xc1, 0xc1, 0xc1, 0xc3, 0xe3, 0xff, 0xff, 0x7e, 0x3c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [140] '*Ś'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xe0, 0xf0, 0xf0, 0x78, 0x3a, 0x3b, 0x39, 0x39, 0xf9, 0xf0, 0xf0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x71, 0xf3, 0xf3, 0xe7, 0xc6, 0xc6, 0xc6, 0xce, 0xee, 0xfc, 0xfc, 0x78, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [141] '*ś'
   0x00, 0xc0, 0xc0, 0xc0, 0xc4, 0xcc, 0xd8, 0xd0, 0xd8, 0xcc, 0xc4, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0xc1, 0xe1, 0xf9, 0xfd, 0x3f, 0x1f, 0x07, 0x03, 0x00, 0x00,
   0x00, 0xc0, 0xe0, 0xf8, 0xfc, 0xff, 0xdf, 0xc7, 0xc1, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, // [142] '*Ž'
   0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xd0, 0xd8, 0xcc, 0xcc, 0xcc, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0xc1, 0xe1, 0xf9, 0xfd, 0x3f, 0x1f, 0x07, 0x03, 0x00, 0x00,
   0x00, 0xc0, 0xe0, 0xf8, 0xfc, 0xff, 0xdf, 0xc7, 0xc1, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, // [143] '*Ź'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x38, 0x38, 0x38, 0x38, 0x3a, 0x3b, 0xb9, 0xf9, 0xf9, 0xf8, 0x78, 0x38, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xf0, 0xf8, 0xfc, 0xfe, 0xdf, 0xcf, 0xc3, 0xc1, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [144] '*ź'
   0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xcc, 0xde, 0xde, 0xcc, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0xc1, 0xe1, 0xf9, 0xfd, 0x3f, 0x1f, 0x07, 0x03, 0x00, 0x00,
   0x00, 0xc0, 0xe0, 0xf8, 0xfc, 0xff, 0xdf, 0xc7, 0xc1, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00,
   0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, // [145] '*Ż'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x38, 0x38, 0x38, 0x38, 0x39, 0x3b, 0xbb, 0xf9, 0xf8, 0xf8, 0x78, 0x38, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xf0, 0xf8, 0xfc, 0xfe, 0xdf, 0xcf, 0xc3, 0xc1, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [146] '*ż'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x38, 0x38, 0x38, 0x38, 0xf8, 0xf8, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [147] '*ı'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xcc, 0xde, 0xde, 0xde, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [148] '*İ'
   0x00, 0x00, 0x00, 0x80, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0x7f, 0xff, 0xf3, 0xe1, 0xc1, 0xc1, 0xc1, 0x83, 0x8f, 0x0f, 0x0f, 0x0c, 0x00,
   0x00, 0x18, 0x78, 0xf8, 0xf8, 0xe0, 0xc1, 0xc1, 0xc1, 0xc3, 0xe3, 0xff, 0xff, 0x7e, 0x3c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x19, 0x1d, 0x1f, 0x0f, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [149] '*Ş'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xe0, 0xf0, 0xf0, 0x78, 0x38, 0x38, 0x38, 0x38, 0xf8, 0xf0, 0xf0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x71, 0xf3, 0xf3, 0xe7, 0xc6, 0xc6, 0xc6, 0xce, 0xee, 0xfc, 0xfc, 0x78, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x19, 0x1d, 0x0f, 0x07, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [150] '*ş'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xc4, 0xcc, 0xd8, 0xd0, 0xd8, 0xcc, 0x84, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x81, 0x81, 0x83, 0x8f, 0x8f, 0x8e, 0x00, 0x00,
   0x00, 0x0f, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc3, 0xc3, 0xc3, 0xff, 0xff, 0x7f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [151] '*Ğ'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xe0, 0xf0, 0xf0, 0x78, 0x39, 0x3b, 0x3a, 0x33, 0xf1, 0xf8, 0xf8, 0xf8, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x08, 0x1c, 0x1c, 0x39, 0x39, 0x39, 0x39, 0x3c, 0x1f, 0x1f, 0x0f, 0x03, 0x00, 0x00, // [152] '*ğ'
   0x00, 0x00, 0xc0, 0xc0, 0xc4, 0xcc, 0xd8, 0xd0, 0xd8, 0xcc, 0xc4, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x81, 0x81, 0x81, 0x81, 0x81, 0xc3, 0xff, 0xff, 0x7f, 0x18, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x03, 0x03, 0x03, 0x07, 0x1f, 0x7f, 0xfd, 0xf0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, // [153] '*Ř'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xe0, 0xf0, 0xf0, 0x78, 0x39, 0x3b, 0x3a, 0x3b, 0xf9, 0xf0, 0xf0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x71, 0xf3, 0xf3, 0xe7, 0xc6, 0xc6, 0xc6, 0xce, 0xee, 0xfc, 0xfc, 0x78, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [154] '*š'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xf8, 0xf8, 0xf9, 0xe3, 0xf2, 0x73, 0x79, 0x78, 0x78, 0x78, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [155] '*ř'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc4, 0xcc, 0xd8, 0xd0, 0xd8, 0xcc, 0xc4, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0x01, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [156] '*Ě'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0xf0, 0x78, 0x39, 0x3b, 0x3a, 0x3b, 0x79, 0xf0, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0x7f, 0xff, 0xf7, 0xc7, 0xc7, 0xc7, 0xc7, 0xc7, 0xe7, 0xe7, 0x47, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [157] '*ě'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x38, 0x38, 0x38, 0x38, 0x39, 0x3b, 0xba, 0xfb, 0xf9, 0xf8, 0x78, 0x38, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xf0, 0xf8, 0xfc, 0xfe, 0xdf, 0xcf, 0xc3, 0xc1, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [158] '*ž'
   0x40, 0xc0, 0xc0, 0xc0, 0xc0, 0x18, 0x18, 0x00, 0x00, 0x18, 0x18, 0xc0, 0xc0, 0xc0, 0xc0, 0x40,
   0x00, 0x00, 0x03, 0x1f, 0x7f, 0xfe, 0xf8, 0xe0, 0xe0, 0xf8, 0xfe, 0x7f, 0x1f, 0x03, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0xff, 0xff, 0xff, 0xff, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [159] '*Ÿ'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x0c, 0x32, 0x21, 0x21, 0x32, 0xcc, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xff, 0xff, 0x7f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [160] '*Ů'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x40, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xf8, 0xf8, 0xf8, 0x00, 0x01, 0x02, 0x02, 0x01, 0xf8, 0xf8, 0xf8, 0xf8, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xf0, 0xc0, 0xc0, 0xc0, 0xc0, 0x7f, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [161] '*ů'
   0x00, 0x00, 0x00, 0x00, 0x90, 0xd8, 0xcc, 0xcc, 0xd0, 0xd8, 0x8c, 0x8c, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x03, 0x07, 0xff, 0xff, 0xfe, 0xe0, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf0, 0xff, 0x7f, 0x3f, 0x03, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [162] '*Ő'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xf0, 0xf2, 0x7b, 0x39, 0x39, 0x3a, 0x3b, 0xf1, 0xf1, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xff, 0x7f, 0x3f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [163] '*ő'
   0x00, 0x00, 0xc0, 0xc0, 0xd0, 0x18, 0x0c, 0x0c, 0x10, 0x18, 0xcc, 0xcc, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xff, 0xff, 0x7f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [164] '*Ű'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xf8, 0xf8, 0xfa, 0x03, 0x01, 0x01, 0x02, 0x03, 0xf9, 0xf9, 0xf8, 0xf8, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xf0, 0xc0, 0xc0, 0xc0, 0xc0, 0x7f, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [165] '*ű'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [166] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [167] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [168] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [169] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [170] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [171] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [172] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [173] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [174] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [175] 'FREE'
   0x00, 0x00, 0x70, 0xd8, 0x8c, 0x04, 0x8c, 0xd8, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [176] '°'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [177] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [178] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [179] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [180] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [181] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [182] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [183] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [184] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [185] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [186] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [187] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [188] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [189] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [190] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [191] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xcc, 0xcc, 0xd8, 0xd0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x80, 0xf0, 0xfe, 0xff, 0x1f, 0x1f, 0xff, 0xfe, 0xf0, 0x80, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0xfc, 0xff, 0xff, 0x1f, 0x1d, 0x1c, 0x1c, 0x1d, 0x1f, 0xff, 0xff, 0xfc, 0xe0, 0x00,
   0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, // [192] 'À'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xd0, 0xd8, 0xcc, 0xcc, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x80, 0xf0, 0xfe, 0xff, 0x1f, 0x1f, 0xff, 0xfe, 0xf0, 0x80, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0xfc, 0xff, 0xff, 0x1f, 0x1d, 0x1c, 0x1c, 0x1d, 0x1f, 0xff, 0xff, 0xfc, 0xe0, 0x00,
   0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, // [193] 'Á'
   0x00, 0x00, 0x00, 0x00, 0x10, 0x18, 0xcc, 0xc6, 0xc6, 0xcc, 0x18, 0x10, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x80, 0xf0, 0xfe, 0xff, 0x1f, 0x1f, 0xff, 0xfe, 0xf0, 0x80, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0xfc, 0xff, 0xff, 0x1f, 0x1d, 0x1c, 0x1c, 0x1d, 0x1f, 0xff, 0xff, 0xfc, 0xe0, 0x00,
   0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, // [194] 'Â'
   0x00, 0x00, 0x00, 0x20, 0x30, 0x18, 0xc8, 0xc8, 0xd8, 0xd0, 0x10, 0x18, 0x08, 0x08, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x80, 0xf0, 0xfe, 0xff, 0x1f, 0x1f, 0xff, 0xfe, 0xf0, 0x80, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0xfc, 0xff, 0xff, 0x1f, 0x1d, 0x1c, 0x1c, 0x1d, 0x1f, 0xff, 0xff, 0xfc, 0xe0, 0x00,
   0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, // [195] 'Ã'
   0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0xc0, 0xc0, 0xc0, 0xc0, 0x18, 0x18, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x80, 0xf0, 0xfe, 0xff, 0x1f, 0x1f, 0xff, 0xfe, 0xf0, 0x80, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0xfc, 0xff, 0xff, 0x1f, 0x1d, 0x1c, 0x1c, 0x1d, 0x1f, 0xff, 0xff, 0xfc, 0xe0, 0x00,
   0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, // [196] 'Ä'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0xe4, 0xc2, 0xc2, 0xe4, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x80, 0xf0, 0xfe, 0xff, 0x1f, 0x1f, 0xff, 0xfe, 0xf0, 0x80, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0xfc, 0xff, 0xff, 0x1f, 0x1d, 0x1c, 0x1c, 0x1d, 0x1f, 0xff, 0xff, 0xfc, 0xe0, 0x00,
   0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, // [197] 'Å'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [198] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x01, 0x03, 0x0f, 0x0f, 0x0e, 0x10, 0x00,
   0x00, 0x0f, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf8, 0x78, 0x38, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x19, 0x1d, 0x1f, 0x0f, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [199] 'Ç'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xcc, 0xcc, 0xcc, 0xd8, 0xd0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0x01, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [200] 'È'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xd0, 0xd8, 0xcc, 0xcc, 0xcc, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0x01, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [201] 'É'
   0x00, 0x00, 0xc0, 0xc0, 0xd0, 0xd8, 0xcc, 0xc6, 0xc6, 0xcc, 0xd8, 0xd0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0x01, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [202] 'Ê'
   0x00, 0x00, 0xc0, 0xc0, 0xd8, 0xd8, 0xc0, 0xc0, 0xc0, 0xc0, 0xd8, 0xd8, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0x01, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [203] 'Ë'
   0x00, 0x00, 0xc0, 0xc0, 0xcc, 0xcc, 0xcc, 0xd8, 0xd0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [204] 'Ì'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xd0, 0xd8, 0xcc, 0xcc, 0xcc, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [205] 'Í'
   0x00, 0x00, 0xc0, 0xc0, 0xd0, 0xd8, 0xcc, 0xc6, 0xc6, 0xcc, 0xd8, 0xd0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [206] 'Î'
   0x00, 0x00, 0xc0, 0xc0, 0xd8, 0xd8, 0xc0, 0xc0, 0xc0, 0xd8, 0xd8, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, // [207] 'Ï'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xc0, 0xff, 0xff, 0xff, 0xc1, 0xc1, 0xc1, 0xc1, 0x03, 0x0f, 0xff, 0xfe, 0xfc, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf8, 0x7f, 0x3f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [208] 'Ð'
   0x00, 0x00, 0xd0, 0xd8, 0xcc, 0x84, 0x04, 0x0c, 0x08, 0x08, 0xcc, 0xc4, 0xc4, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x3f, 0xfe, 0xf0, 0xc0, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x03, 0x1f, 0x7f, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [209] 'Ñ'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xcc, 0xcc, 0xcc, 0xd8, 0xd0, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x03, 0x07, 0xff, 0xff, 0xfe, 0xe0, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf0, 0xff, 0x7f, 0x3f, 0x03, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [210] 'Ò'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xd0, 0xd8, 0xcc, 0xcc, 0x8c, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x03, 0x07, 0xff, 0xff, 0xfe, 0xe0, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf0, 0xff, 0x7f, 0x3f, 0x03, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [211] 'Ó'
   0x00, 0x00, 0x00, 0x00, 0x90, 0xd8, 0xcc, 0xc6, 0xc6, 0xcc, 0x98, 0x90, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x03, 0x07, 0xff, 0xff, 0xfe, 0xe0, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf0, 0xff, 0x7f, 0x3f, 0x03, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [212] 'Ô'
   0x00, 0x00, 0x20, 0x30, 0x98, 0xc8, 0xc8, 0xd8, 0xd0, 0xd0, 0x98, 0x88, 0x08, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x03, 0x07, 0xff, 0xff, 0xfe, 0xe0, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf0, 0xff, 0x7f, 0x3f, 0x03, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [213] 'Õ'
   0x00, 0x00, 0x00, 0x00, 0x98, 0xd8, 0xc0, 0xc0, 0xc0, 0xd8, 0x98, 0x80, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0x01, 0x01, 0x03, 0x07, 0xff, 0xff, 0xfe, 0xe0, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xf0, 0xff, 0x7f, 0x3f, 0x03, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [214] 'Ö'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [215] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0xc0, 0x40, 0x00, 0x00,
   0x00, 0xf8, 0xfe, 0xff, 0xff, 0x03, 0x01, 0xc1, 0xf1, 0x3b, 0x0f, 0xff, 0xff, 0xfe, 0xe0, 0x00,
   0x00, 0x0f, 0xbf, 0xff, 0xff, 0xec, 0xc7, 0xc1, 0xc0, 0xe0, 0xf0, 0xff, 0x7f, 0x3f, 0x03, 0x00,
   0x00, 0x00, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [216] 'Ø'
   0x00, 0x00, 0xc0, 0xc0, 0xcc, 0x0c, 0x0c, 0x18, 0x10, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xff, 0xff, 0x7f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [217] 'Ù'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0x00, 0x10, 0x18, 0x0c, 0x0c, 0xcc, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xff, 0xff, 0x7f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [218] 'Ú'
   0x00, 0x00, 0xc0, 0xc0, 0xd0, 0x18, 0x0c, 0x06, 0x06, 0x0c, 0xd8, 0xd0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xff, 0xff, 0x7f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [219] 'Û'
   0x00, 0x00, 0xc0, 0xc0, 0xd8, 0x18, 0x00, 0x00, 0x00, 0x18, 0xd8, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xe0, 0xff, 0xff, 0x7f, 0x1f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [220] 'Ü'
   0x40, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x10, 0x18, 0x0c, 0x0c, 0x0c, 0xc0, 0xc0, 0xc0, 0xc0, 0x40,
   0x00, 0x00, 0x03, 0x1f, 0x7f, 0xfe, 0xf8, 0xe0, 0xe0, 0xf8, 0xfe, 0x7f, 0x1f, 0x03, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0xff, 0xff, 0xff, 0xff, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [221] 'Ý'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [222] 'FREE'
   0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x71, 0x71, 0xfb, 0xff, 0xdf, 0xcf, 0x02, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xff, 0xff, 0x7f, 0x3f, 0x00,
   0x00, 0x00, 0x1f, 0x0f, 0x01, 0x00, 0x00, 0x02, 0x03, 0x03, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, // [223] 'ß'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x60, 0x70, 0x70, 0x39, 0x39, 0x39, 0x3b, 0x7a, 0xf0, 0xf0, 0xe0, 0x00, 0x00, 0x00,
   0x00, 0x10, 0xfc, 0xfe, 0xfe, 0xc7, 0xc3, 0xc3, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [224] 'à'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x60, 0x70, 0x70, 0x3a, 0x3b, 0x39, 0x39, 0x79, 0xf0, 0xf0, 0xe0, 0x00, 0x00, 0x00,
   0x00, 0x10, 0xfc, 0xfe, 0xfe, 0xc7, 0xc3, 0xc3, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [225] 'á'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x60, 0x70, 0x72, 0x3b, 0x39, 0x38, 0x38, 0x79, 0xf3, 0xf2, 0xe0, 0x00, 0x00, 0x00,
   0x00, 0x10, 0xfc, 0xfe, 0xfe, 0xc7, 0xc3, 0xc3, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [226] 'â'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x64, 0x76, 0x73, 0x39, 0x39, 0x3b, 0x3a, 0x7a, 0xf3, 0xf1, 0xe1, 0x00, 0x00, 0x00,
   0x00, 0x10, 0xfc, 0xfe, 0xfe, 0xc7, 0xc3, 0xc3, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [227] 'ã'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x60, 0x70, 0x73, 0x3b, 0x38, 0x38, 0x38, 0x7b, 0xf3, 0xf0, 0xe0, 0x00, 0x00, 0x00,
   0x00, 0x10, 0xfc, 0xfe, 0xfe, 0xc7, 0xc3, 0xc3, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [228] 'ä'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x40, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x60, 0x70, 0x70, 0x38, 0x39, 0x3a, 0x3a, 0x79, 0xf0, 0xf0, 0xe0, 0x00, 0x00, 0x00,
   0x00, 0x10, 0xfc, 0xfe, 0xfe, 0xc7, 0xc3, 0xc3, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [229] 'å'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [230] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0xf0, 0x78, 0x38, 0x38, 0x38, 0x38, 0xf8, 0xf0, 0xe0, 0x80, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0x7f, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xf0, 0x70, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x19, 0x1d, 0x0f, 0x07, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [231] 'ç'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0xf0, 0x79, 0x39, 0x39, 0x3b, 0x3a, 0x78, 0xf0, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0x7f, 0xff, 0xf7, 0xc7, 0xc7, 0xc7, 0xc7, 0xc7, 0xe7, 0xe7, 0x47, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [232] 'è'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0xf0, 0x78, 0x3a, 0x3b, 0x39, 0x39, 0x79, 0xf0, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0x7f, 0xff, 0xf7, 0xc7, 0xc7, 0xc7, 0xc7, 0xc7, 0xe7, 0xe7, 0x47, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [233] 'é'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0xf2, 0x7b, 0x39, 0x38, 0x38, 0x39, 0x7b, 0xf2, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0x7f, 0xff, 0xf7, 0xc7, 0xc7, 0xc7, 0xc7, 0xc7, 0xe7, 0xe7, 0x47, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [234] 'ê'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xe0, 0xf0, 0x7b, 0x3b, 0x38, 0x38, 0x38, 0x7b, 0xf3, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x06, 0x3f, 0x7f, 0xff, 0xf7, 0xc7, 0xc7, 0xc7, 0xc7, 0xc7, 0xe7, 0xe7, 0x47, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [235] 'ë'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x38, 0x38, 0x39, 0x39, 0xf9, 0xfb, 0xfa, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [236] 'ì'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x38, 0x38, 0x3a, 0x3b, 0xf9, 0xf9, 0xf9, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [237] 'í'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x3a, 0x3b, 0x39, 0x38, 0xf8, 0xf9, 0xfb, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [238] 'î'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x3b, 0x3b, 0x38, 0x38, 0xf8, 0xfb, 0xfb, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [239] 'ï'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0xe0, 0xe0, 0xe0, 0xe0, 0x80, 0x00,
   0x00, 0x00, 0xe0, 0xf0, 0xf0, 0x78, 0x38, 0x38, 0x39, 0x31, 0xff, 0xff, 0xff, 0xff, 0x01, 0x00,
   0x00, 0x0f, 0x7f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, // [240] 'ð'
   0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xfa, 0xfb, 0xf9, 0x60, 0x30, 0x39, 0x39, 0x39, 0xf9, 0xf8, 0xf0, 0xe0, 0x00, 0x00,
   0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [241] 'ñ'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xf0, 0xf0, 0x79, 0x39, 0x39, 0x3b, 0x3a, 0xf0, 0xf0, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xff, 0x7f, 0x3f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [242] 'ò'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xf0, 0xf0, 0x7a, 0x3b, 0x39, 0x39, 0x39, 0xf0, 0xf0, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xff, 0x7f, 0x3f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [243] 'ó'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xf0, 0xf2, 0x7b, 0x39, 0x38, 0x38, 0x39, 0xf3, 0xf2, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xff, 0x7f, 0x3f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [244] 'ô'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc4, 0xf6, 0xf3, 0x79, 0x39, 0x3b, 0x3a, 0x3a, 0xf3, 0xf1, 0xe1, 0xc0, 0x00, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xff, 0x7f, 0x3f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [245] 'õ'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xf0, 0xf3, 0x7b, 0x38, 0x38, 0x38, 0x3b, 0xf3, 0xf0, 0xe0, 0xc0, 0x00, 0x00,
   0x00, 0x0f, 0x3f, 0xff, 0xff, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xf0, 0xff, 0x7f, 0x3f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [246] 'ö'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [247] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xc0, 0xf0, 0xf0, 0x78, 0x38, 0x38, 0xb8, 0xf8, 0xf0, 0xf8, 0xf8, 0xd8, 0x00, 0x00,
   0x00, 0x0f, 0xbf, 0xff, 0xff, 0xf8, 0xdc, 0xcf, 0xc7, 0xc1, 0xf0, 0xff, 0x7f, 0x3f, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, // [248] 'ø'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xf8, 0xf8, 0xf8, 0x01, 0x01, 0x01, 0x03, 0x02, 0xf8, 0xf8, 0xf8, 0xf8, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xf0, 0xc0, 0xc0, 0xc0, 0xc0, 0x7f, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [249] 'ù'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xf8, 0xf8, 0xf8, 0x02, 0x03, 0x01, 0x01, 0x01, 0xf8, 0xf8, 0xf8, 0xf8, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xf0, 0xc0, 0xc0, 0xc0, 0xc0, 0x7f, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [250] 'ú'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xf8, 0xf8, 0xfa, 0x03, 0x01, 0x00, 0x00, 0x01, 0xfb, 0xfa, 0xf8, 0xf8, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xf0, 0xc0, 0xc0, 0xc0, 0xc0, 0x7f, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [251] 'û'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xf8, 0xf8, 0xfb, 0x03, 0x00, 0x00, 0x00, 0x03, 0xfb, 0xf8, 0xf8, 0xf8, 0x00, 0x00,
   0x00, 0x00, 0x3f, 0xff, 0xff, 0xf0, 0xc0, 0xc0, 0xc0, 0xc0, 0x7f, 0xff, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, // [252] 'ü'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x18, 0x78, 0xf8, 0xf8, 0xe0, 0x02, 0x03, 0x01, 0x01, 0xc1, 0xf8, 0xf8, 0xf8, 0x38, 0x08,
   0x00, 0x00, 0x00, 0x01, 0x0f, 0x3f, 0xff, 0xf8, 0xf8, 0xfe, 0x3f, 0x0f, 0x03, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x30, 0x38, 0x38, 0x3c, 0x1f, 0x0f, 0x07, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [253] 'ý'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // [254] 'FREE'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x18, 0x78, 0xf8, 0xf8, 0xe3, 0x03, 0x00, 0x00, 0x03, 0xc3, 0xf8, 0xf8, 0xf8, 0x38, 0x08,
   0x00, 0x00, 0x00, 0x01, 0x0f, 0x3f, 0xff, 0xf8, 0xf8, 0xfe, 0x3f, 0x0f, 0x03, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x30, 0x38, 0x38, 0x3c, 0x1f, 0x0f, 0x07, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  // [255] 'ÿ'
};


/*
// This AtariST style font, supplied by Jason VK7ZJA probably can't be used
// because of its unknown origin and license
const uint8_t font_AtariST_8x16[] = {
   0x00, 0x00, // Ignored.
   32        , // first char ASCII code.
   126       , // last char ASCII code.
   8         , // width of the character in pixels.
   16        , // height of the character in pixels.
   1         , // number of line of bytes per character.
   16        , // bytes per character (normally width * height / 8).
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // ' '
   0x00, 0x00, 0x00, 0xFC, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x33, 0x33, 0x00, 0x00, 0x00, // '!'
   0x00, 0xFC, 0xFC, 0x00, 0x00, 0xFC, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // '"'
   0x30, 0xFC, 0xFC, 0x30, 0x30, 0xFC, 0xFC, 0x30, 0x03, 0x0F, 0x0F, 0x03, 0x03, 0x0F, 0x0F, 0x03, // '#'
   0x00, 0x78, 0xFC, 0xCF, 0xCF, 0xCC, 0x8C, 0x00, 0x00, 0x0C, 0x0C, 0x3C, 0x3C, 0x0F, 0x07, 0x00, // '$'
   0x00, 0x1C, 0x1C, 0xC0, 0xF0, 0x3C, 0x0C, 0x00, 0x00, 0x0C, 0x0F, 0x03, 0x00, 0x0E, 0x0E, 0x00, // '%'
   0x00, 0xCE, 0xFF, 0xF3, 0x33, 0x0E, 0x0C, 0x00, 0x1F, 0x3F, 0x30, 0x33, 0x13, 0x3F, 0x3F, 0x00, // '&'
   0x00, 0x00, 0x00, 0xFC, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // '''
   0x00, 0x00, 0x00, 0xF0, 0xF8, 0x1C, 0x04, 0x00, 0x00, 0x00, 0x00, 0x0F, 0x1F, 0x38, 0x20, 0x00, // '('
   0x00, 0x04, 0x1C, 0xF8, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x20, 0x38, 0x1F, 0x0F, 0x00, 0x00, 0x00, // ')'
   0xC0, 0xCC, 0xFC, 0xF0, 0xF0, 0xFC, 0xCC, 0xC0, 0x00, 0x0C, 0x0F, 0x03, 0x03, 0x0F, 0x0C, 0x00, // '*'
   0x00, 0xC0, 0xC0, 0xF8, 0xF8, 0xC0, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x07, 0x07, 0x00, 0x00, 0x00, // '+'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0x7C, 0x3C, 0x00, 0x00, 0x00, // ','
   0x00, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // '-'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3C, 0x3C, 0x00, 0x00, 0x00, // '.'
   0x00, 0x00, 0x00, 0x80, 0xE0, 0x7C, 0x1C, 0x00, 0x00, 0x38, 0x3E, 0x07, 0x01, 0x00, 0x00, 0x00, // '/'
   0x00, 0xF8, 0xFC, 0x0C, 0x8C, 0xFC, 0xF8, 0x00, 0x00, 0x1F, 0x3F, 0x31, 0x30, 0x3F, 0x1F, 0x00, // '0'
   0x00, 0x00, 0x30, 0xFC, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x30, 0x30, 0x3F, 0x3F, 0x30, 0x30, 0x00, // '1'
   0x00, 0x38, 0x3C, 0x0C, 0xCC, 0xFC, 0x38, 0x00, 0x00, 0x30, 0x3C, 0x3F, 0x33, 0x30, 0x30, 0x00, // '2'
   0x00, 0x0C, 0x0C, 0xCC, 0xFC, 0x3C, 0x0C, 0x00, 0x00, 0x1C, 0x3C, 0x30, 0x33, 0x3F, 0x1C, 0x00, // '3'
   0x00, 0x00, 0xC0, 0xF0, 0xFC, 0xFC, 0x00, 0x00, 0x00, 0x0F, 0x0F, 0x0C, 0x3F, 0x3F, 0x0C, 0x00, // '4'
   0x00, 0xFC, 0xFC, 0xCC, 0xCC, 0xCC, 0x8C, 0x00, 0x00, 0x18, 0x38, 0x30, 0x30, 0x3F, 0x1F, 0x00, // '5'
   0x00, 0xF0, 0xF8, 0x9C, 0x8C, 0x8C, 0x00, 0x00, 0x00, 0x1F, 0x3F, 0x31, 0x31, 0x3F, 0x1F, 0x00, // '6'
   0x00, 0x0C, 0x0C, 0x0C, 0xCC, 0xFC, 0x3C, 0x00, 0x00, 0x00, 0x3C, 0x3F, 0x03, 0x00, 0x00, 0x00, // '7'
   0x00, 0x38, 0xFC, 0xCC, 0xCC, 0xFC, 0x38, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x30, 0x3F, 0x1F, 0x00, // '8'
   0x00, 0x78, 0xFC, 0xCC, 0xCC, 0xFC, 0xF8, 0x00, 0x00, 0x00, 0x30, 0x30, 0x38, 0x1F, 0x0F, 0x00, // '9'
   0x00, 0x00, 0x00, 0xF0, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3C, 0x3C, 0x00, 0x00, 0x00, // ':'
   0x00, 0x00, 0x00, 0xF0, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0x7C, 0x3C, 0x00, 0x00, 0x00, // ';'
   0x80, 0xC0, 0xE0, 0x70, 0x38, 0x18, 0x08, 0x00, 0x00, 0x01, 0x03, 0x07, 0x0E, 0x0C, 0x08, 0x00, // '<'
   0x00, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x00, 0x00, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x00, // '='
   0x08, 0x18, 0x38, 0x70, 0xE0, 0xC0, 0x80, 0x00, 0x08, 0x0C, 0x0E, 0x07, 0x03, 0x01, 0x00, 0x00, // '>'
   0x00, 0x38, 0x3C, 0x0C, 0xCC, 0xFC, 0x38, 0x00, 0x00, 0x00, 0x00, 0x37, 0x37, 0x00, 0x00, 0x00, // '?'
   0xF0, 0xF8, 0x1C, 0xCC, 0x4C, 0x98, 0xF0, 0x00, 0x0F, 0x1F, 0x38, 0x33, 0x32, 0x33, 0x19, 0x00, // '@'
   0x00, 0xF0, 0xF8, 0x1C, 0x1C, 0xF8, 0xF0, 0x00, 0x00, 0x3F, 0x3F, 0x03, 0x03, 0x3F, 0x3F, 0x00, // 'A'
   0x00, 0xFC, 0xFC, 0xCC, 0xCC, 0xFC, 0x78, 0x00, 0x00, 0x3F, 0x3F, 0x30, 0x30, 0x3F, 0x1F, 0x00, // 'B'
   0x00, 0xF8, 0xFC, 0x0C, 0x0C, 0x3C, 0x38, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x30, 0x3C, 0x1C, 0x00, // 'C'
   0x00, 0xFC, 0xFC, 0x0C, 0x1C, 0xF8, 0xF0, 0x00, 0x00, 0x3F, 0x3F, 0x30, 0x38, 0x1F, 0x0F, 0x00, // 'D'
   0x00, 0xFC, 0xFC, 0xCC, 0xCC, 0xCC, 0x0C, 0x00, 0x00, 0x3F, 0x3F, 0x30, 0x30, 0x30, 0x30, 0x00, // 'E'
   0x00, 0xFC, 0xFC, 0xCC, 0xCC, 0xCC, 0x0C, 0x00, 0x00, 0x3F, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, // 'F'
   0x00, 0xF8, 0xFC, 0x0C, 0xCC, 0xCC, 0xCC, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x30, 0x3F, 0x1F, 0x00, // 'G'
   0x00, 0xFC, 0xFC, 0xC0, 0xC0, 0xFC, 0xFC, 0x00, 0x00, 0x3F, 0x3F, 0x00, 0x00, 0x3F, 0x3F, 0x00, // 'H'
   0x00, 0x0C, 0x0C, 0xFC, 0xFC, 0x0C, 0x0C, 0x00, 0x00, 0x30, 0x30, 0x3F, 0x3F, 0x30, 0x30, 0x00, // 'I'
   0x00, 0x00, 0x00, 0x00, 0x00, 0xFC, 0xFC, 0x00, 0x00, 0x1C, 0x3C, 0x30, 0x30, 0x3F, 0x1F, 0x00, // 'J'
   0xFC, 0xFC, 0xC0, 0xF0, 0x3C, 0x0C, 0x00, 0x00, 0x3F, 0x3F, 0x00, 0x03, 0x0F, 0x3C, 0x30, 0x00, // 'K'
   0x00, 0xFC, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x3F, 0x30, 0x30, 0x30, 0x30, 0x00, // 'L'
   0xFC, 0xFC, 0x70, 0xC0, 0x70, 0xFC, 0xFC, 0x00, 0x3F, 0x3F, 0x00, 0x01, 0x00, 0x3F, 0x3F, 0x00, // 'M'
   0x00, 0xFC, 0xFC, 0xE0, 0x80, 0xFC, 0xFC, 0x00, 0x00, 0x3F, 0x3F, 0x01, 0x07, 0x3F, 0x3F, 0x00, // 'N'
   0x00, 0xF8, 0xFC, 0x0C, 0x0C, 0xFC, 0xF8, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x30, 0x3F, 0x1F, 0x00, // 'O'
   0x00, 0xFC, 0xFC, 0x0C, 0x0C, 0xFC, 0xF8, 0x00, 0x00, 0x3F, 0x3F, 0x03, 0x03, 0x03, 0x01, 0x00, // 'P'
   0x00, 0xF8, 0xFC, 0x0C, 0x0C, 0xFC, 0xF8, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x18, 0x37, 0x2F, 0x00, // 'Q'
   0xFC, 0xFC, 0x8C, 0x8C, 0xFC, 0xF8, 0x00, 0x00, 0x3F, 0x3F, 0x01, 0x03, 0x0F, 0x3C, 0x30, 0x00, // 'R'
   0x00, 0x78, 0xFC, 0xCC, 0x8C, 0x0C, 0x0C, 0x00, 0x00, 0x30, 0x30, 0x31, 0x33, 0x3F, 0x1E, 0x00, // 'S'
   0x00, 0x0C, 0x0C, 0xFC, 0xFC, 0x0C, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x3F, 0x00, 0x00, 0x00, // 'T'
   0x00, 0xFC, 0xFC, 0x00, 0x00, 0xFC, 0xFC, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x30, 0x3F, 0x1F, 0x00, // 'U'
   0x00, 0xFC, 0xFC, 0x00, 0x00, 0xFC, 0xFC, 0x00, 0x00, 0x03, 0x0F, 0x3C, 0x3C, 0x0F, 0x03, 0x00, // 'V'
   0xFC, 0xFC, 0x00, 0x80, 0x00, 0xFC, 0xFC, 0x00, 0x3F, 0x1F, 0x0E, 0x07, 0x0E, 0x1F, 0x3F, 0x00, // 'W'
   0x00, 0x1C, 0x7C, 0xE0, 0xE0, 0x7C, 0x1C, 0x00, 0x00, 0x38, 0x3E, 0x07, 0x07, 0x3E, 0x38, 0x00, // 'X'
   0x00, 0x3C, 0xFC, 0xC0, 0xC0, 0xFC, 0x3C, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x3F, 0x00, 0x00, 0x00, // 'Y'
   0x00, 0x0C, 0x0C, 0xCC, 0xFC, 0x3C, 0x0C, 0x00, 0x00, 0x3C, 0x3F, 0x33, 0x30, 0x30, 0x30, 0x00, // 'Z'
   0x00, 0x00, 0x00, 0xFC, 0xFC, 0x0C, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x3F, 0x30, 0x30, 0x00, // '['
   0x00, 0x1C, 0x7C, 0xE0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x07, 0x3E, 0x38, 0x00, // '\'
   0x00, 0x0C, 0x0C, 0xFC, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x30, 0x30, 0x3F, 0x3F, 0x00, 0x00, 0x00, // ']'
   0x80, 0xE0, 0x78, 0x1E, 0x78, 0xE0, 0x80, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, // '^'
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 0x00, // '_'
   0x00, 0x07, 0x0E, 0x1C, 0x38, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // '`'
   0x00, 0x00, 0x60, 0x60, 0x60, 0xE0, 0xC0, 0x00, 0x00, 0x1E, 0x3F, 0x33, 0x33, 0x3F, 0x3F, 0x00, // 'a'
   0x00, 0xFC, 0xFC, 0x60, 0x60, 0xE0, 0xC0, 0x00, 0x00, 0x3F, 0x3F, 0x30, 0x30, 0x3F, 0x1F, 0x00, // 'b'
   0x00, 0xC0, 0xE0, 0x20, 0x60, 0x60, 0x00, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x30, 0x30, 0x30, 0x00, // 'c'
   0x00, 0xC0, 0xE0, 0x60, 0x60, 0xFC, 0xFC, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x30, 0x3F, 0x3F, 0x00, // 'd'
   0x00, 0xC0, 0xE0, 0x60, 0x60, 0xE0, 0xC0, 0x00, 0x00, 0x1F, 0x3F, 0x32, 0x32, 0x33, 0x33, 0x00, // 'e'
   0x00, 0xC0, 0xC0, 0xF8, 0xFC, 0xCC, 0xCC, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x3F, 0x00, 0x00, 0x00, // 'f'
   0x00, 0xC0, 0xE0, 0x60, 0x60, 0xE0, 0xE0, 0x00, 0x00, 0xCF, 0xDF, 0xD8, 0xD8, 0xFF, 0x7F, 0x00, // 'g'
   0x00, 0xFC, 0xFC, 0x60, 0x60, 0xE0, 0xC0, 0x00, 0x00, 0x3F, 0x3F, 0x00, 0x00, 0x3F, 0x3F, 0x00, // 'h'
   0x00, 0x00, 0x60, 0xEC, 0xEC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x3F, 0x3F, 0x30, 0x00, 0x00, // 'i'
   0x00, 0x00, 0x00, 0xEC, 0xEC, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xC0, 0xFF, 0x7F, 0x00, 0x00, 0x00, // 'j'
   0xFC, 0xFC, 0x80, 0xC0, 0xE0, 0x60, 0x00, 0x00, 0x3F, 0x3F, 0x03, 0x07, 0x1E, 0x38, 0x30, 0x00, // 'k'
   0x00, 0x00, 0x0C, 0xFC, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x3F, 0x3F, 0x30, 0x00, 0x00, // 'l'
   0xC0, 0xE0, 0xE0, 0xC0, 0xE0, 0xE0, 0xC0, 0x00, 0x3F, 0x3F, 0x00, 0x07, 0x00, 0x3F, 0x3F, 0x00, // 'm'
   0x00, 0xC0, 0xE0, 0x60, 0x60, 0xE0, 0xC0, 0x00, 0x00, 0x3F, 0x3F, 0x00, 0x00, 0x3F, 0x3F, 0x00, // 'n'
   0x00, 0xC0, 0xE0, 0x60, 0x60, 0xE0, 0xC0, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x30, 0x3F, 0x1F, 0x00, // 'o'
   0x00, 0xE0, 0xE0, 0x60, 0x60, 0xE0, 0xC0, 0x00, 0x00, 0xFF, 0xFF, 0x30, 0x30, 0x3F, 0x1F, 0x00, // 'p'
   0x00, 0xC0, 0xE0, 0x60, 0x60, 0xE0, 0xE0, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x30, 0xFF, 0xFF, 0x00, // 'q'
   0x00, 0xE0, 0xE0, 0x60, 0x60, 0xE0, 0xC0, 0x00, 0x00, 0x3F, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00, // 'r'
   0x00, 0xC0, 0xE0, 0x60, 0x60, 0x60, 0x60, 0x00, 0x00, 0x31, 0x33, 0x33, 0x36, 0x3E, 0x1C, 0x00, // 's'
   0x00, 0x60, 0x60, 0xF8, 0xF8, 0x60, 0x60, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x30, 0x00, // 't'
   0x00, 0xE0, 0xE0, 0x00, 0x00, 0xE0, 0xE0, 0x00, 0x00, 0x1F, 0x3F, 0x30, 0x30, 0x3F, 0x3F, 0x00, // 'u'
   0x00, 0xE0, 0xE0, 0x00, 0x00, 0xE0, 0xE0, 0x00, 0x00, 0x03, 0x0F, 0x3C, 0x3C, 0x0F, 0x03, 0x00, // 'v'
   0xE0, 0xE0, 0x00, 0x80, 0x00, 0xE0, 0xE0, 0x00, 0x3F, 0x1F, 0x0E, 0x07, 0x0E, 0x1F, 0x3F, 0x00, // 'w'
   0x00, 0x60, 0xE0, 0x80, 0x80, 0xE0, 0x60, 0x00, 0x00, 0x30, 0x3D, 0x0F, 0x0F, 0x3D, 0x30, 0x00, // 'x'
   0x00, 0xE0, 0xE0, 0x00, 0x00, 0xE0, 0xE0, 0x00, 0x00, 0x4F, 0x5F, 0x58, 0x58, 0x7F, 0x7F, 0x00, // 'y'
   0x00, 0x60, 0x60, 0x60, 0xE0, 0xE0, 0x60, 0x00, 0x00, 0x30, 0x3C, 0x3F, 0x33, 0x30, 0x30, 0x00, // 'z'
   0x80, 0x80, 0xC0, 0xFC, 0x7E, 0x02, 0x02, 0x00, 0x01, 0x01, 0x03, 0x3F, 0x7E, 0x40, 0x40, 0x00, // '{'
   0x00, 0x00, 0x00, 0xFE, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x7F, 0x00, 0x00, 0x00, // '|'
   0x02, 0x02, 0x7E, 0xFC, 0xC0, 0x80, 0x80, 0x00, 0x40, 0x40, 0x7E, 0x3F, 0x03, 0x01, 0x01, 0x00, // '}'
   0xC0, 0xE0, 0x60, 0xC0, 0x80, 0x80, 0xC0, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00  // '~'
};
*/

#endif // ! PLATFORM_GD77S

#endif // ! CHAR_CONSTANT_ONLY

#endif /* _OPENGD77_UC1701_CHARSET_H_ */
